# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010, Rectorate of the University of Freiburg
# Copyright (c) 2009-2010, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module provides the H5FileHandler class.
"""
import tables
from pyphant.core import DataContainer
from pyphant.quantities import Quantity
PhysicalQuantity = Quantity
import logging
import os
from pyphant.core import PyTablesPersister
from pyphant.core.DataContainer import IndexMarker
from pyphant.core.Helpers import (utf82uc, emd52dict)
_logger = logging.getLogger("pyphant")

im = IndexMarker()
im_id = u"emd5://pyphant/pyphant/0001-01-01_00:00:00.000000/%s.field" \
        % utf82uc(im.hash)
im_summary = {'id':im_id, 'longname':utf82uc(im.longname),
              'shortname':utf82uc(im.shortname), 'hash':utf82uc(im.hash),
              'creator':u'pyphant', 'machine':u'pyphant',
              'date':u'0001-01-01_00:00:00.000000',
              'unit':1, 'dimensions':[im_id], 'attributes':{}}


class H5FileHandler(object):
    """
    This class is used to handle IO operations on HDF5 files.
    """
    def __init__(self, filename, mode='a'):
        """
        Opens an HDF5 file.
        filename -- path to the file that should be opened
        mode -- mode in which file is opened. Possible values: 'r', 'w', 'a'
                meaning 'read only', 'overwrite' and 'append'.
                'a' is only allowed for files that are valid HDF5 files
                already.
        """
        assert mode in ['r', 'w', 'a']
        exists = os.path.isfile(filename)
        if mode == 'r' and not exists:
            raise IOError("File '%s' does not exist!"%(filename, ))
        self.filename = filename
        self.mode = mode
        if mode == 'w':
            tmphandle = tables.openFile(self.filename, 'w')
            tmphandle.close()
            self.mode = 'a'
        self.handle = None

    def __enter__(self):
        assert self.handle ==  None
        self.handle = tables.openFile(self.filename, self.mode)
        return self

    def __exit__(self, type, value, traceback):
        if self.handle != None:
            self.handle.close()
            self.handle = None

    def getNodeAndTypeFromId(self, dcId):
        """
        Returns a tuple (HDF5 node, uriType) for the given
        DataContainer emd5.
        dcId -- emd5 of the DataContainer
        """
        dcHash, uriType = DataContainer.parseId(dcId)
        try:
            resNode = self.handle.getNode("/results/result_" + dcHash)
        except (AttributeError, tables.NoSuchNodeError):
            raise AttributeError("Container %s not found in file %s."
                                 % (dcId, self.filename))
        if isinstance(uriType, unicode):
            uriType = uriType.encode('utf-8')
        return (resNode, uriType)

    def isIndexMarker(self, dcId):
        """
        returns True iff the underlying HDF5 file contains dcId and
        dcId belongs to an IndexMarker
        """
        resNode, uriType = self.getNodeAndTypeFromId(dcId)
        if uriType == u'field':
            try:
                resNode._g_checkHasChild('dimensions')
            except tables.NoSuchNodeError:
                return True
        return False

    def loadDataContainer(self, dcId):
        """
        Loads a DataContainer from the HDF5 file and returns it as a
        DataContainer instance.
        dcId -- emd5 of the DC to be returned
        """
        resNode, uriType = self.getNodeAndTypeFromId(dcId)
        if uriType == 'field':
            result = self.loadField(resNode)
        elif uriType == 'sample':
            result = self.loadSample(resNode)
        else:
            raise TypeError, "Unknown result uriType in <%s>" % resNode._v_title
        return result

    def loadField(self, resNode):
        """
        Loads a FieldContainer from the given node and returns it as an
        instance. This method is intended for internal use only.
        resNode -- node at which the FieldContainer is located in the file.
        """
        return PyTablesPersister.loadField(self.handle, resNode)

    def loadSample(self, resNode):
        """
        Loads a SampleContainer from the given node and returns it as an
        instance. This method is intended for internal use only.
        resNode -- node at which the SampleContainer is located in the file.
        """
        return PyTablesPersister.loadSample(self.handle, resNode)

    def loadSummary(self, dcId = None):
        """
        Extracts meta data about a given DataContainer and returns it
        as a dictionary.
        dcId -- emd5 of the DC to summarize. If the emd5 belongs to an
                IndexMarker object, u'IndexMarker' is returned.
                If dcId == None, a dictionary that maps emd5s to summaries
                is returned, where IndexMarker objects are ignored.
        """
        if dcId == None:
            summary = {}
            for group in self.handle.walkGroups(where = "/results"):
                currDcId = group._v_attrs.TITLE
                if len(currDcId) > 0:
                    tmp = self.loadSummary(currDcId)
                    if tmp == 'IndexMarker':
                        summary[im_id] = im_summary
                    else:
                        summary[currDcId] = tmp
        elif self.isIndexMarker(dcId):
            return u'IndexMarker'
        else:
            summary = {}
            summary['id'] = dcId
            resNode, uriType = self.getNodeAndTypeFromId(dcId)
            summary['longname'] = utf82uc(self.handle.getNodeAttr(resNode,
                                                                  "longname"))
            summary['shortname'] = utf82uc(self.handle.getNodeAttr(resNode,
                                                                   "shortname"))
            summary.update(emd52dict(dcId))
            try:
                summary['machine'] = utf82uc(self.handle.getNodeAttr(resNode,
                                                                     "machine"))
                summary['creator'] = utf82uc(self.handle.getNodeAttr(resNode,
                                                                     "creator"))
            except:
                pass # machine, creator set by emd52dict(dcId) before
            attributes = {}
            if uriType == 'field':
                for key in resNode.data._v_attrs._v_attrnamesuser:
                    attributes[key]=self.handle.getNodeAttr(resNode.data, key)
                unit = eval(utf82uc(self.handle.getNodeAttr(resNode, "unit")))
                summary['unit'] = unit
                dimTable = resNode.dimensions
                def filterIndexMarker(emd5):
                    if self.isIndexMarker(emd5):
                        return im_id
                    else:
                        return emd5
                dimensions = [filterIndexMarker(row['id']) \
                                  for row in dimTable.iterrows()]
                summary['dimensions'] = dimensions
            elif uriType == 'sample':
                for key in resNode._v_attrs._v_attrnamesuser:
                    if key not in PyTablesPersister._reservedAttributes:
                        attributes[key] = self.handle.getNodeAttr(resNode, key)
                columns = []
                for resId in self.handle.getNodeAttr(resNode, "columns"):
                    nodename = "/results/" + resId
                    columnId = self.handle.getNodeAttr(nodename, "TITLE")
                    columns.append(columnId)
                summary['columns'] = columns
            summary['attributes'] = attributes
        return summary

    def loadRecipe(self):
        return PyTablesPersister.loadRecipe(self.handle)

    def saveDataContainer(self, result):
        """
        Saves a given DataContainer instance to the HDF5 file.
        The DataContainer has to be sealed or at least provide a valid
        emd5 in its '.id' attribute.
        A HDF5 group path that points to the location the DC was stored at
        is returned.
        result -- sealed DC instance
        """
        dcHash, uriType = DataContainer.parseId(result.id)
        resId = u"result_" + dcHash
        try:
            resultGroup = self.handle.getNode("/results/"+resId)
        except tables.NoSuchNodeError:
            try:
                resultGroup = self.handle.createGroup("/results", resId,
                                                      result.id.encode("utf-8"))
            except tables.NoSuchNodeError:
                self.handle.createGroup('/', 'results')
                resultGroup = self.handle.createGroup("/results", resId,
                                                      result.id.encode("utf-8"))
            if uriType == 'field':
                self.saveField(resultGroup, result)
            elif uriType == 'sample':
                self.saveSample(resultGroup, result)
            else:
                raise KeyError, "Unknown UriType %s in saving result %s."\
                    % (uriType, result.id)
        return resId

    def saveSample(self, resultGroup, result):
        """
        Saves a SampleContainer instance to the given node. This method is
        intended for internal use only.
        resultGroup -- node at which the SampleContainer should be saved
                       in the file.
        result -- SampleContainer instance to be saved
        """
        return PyTablesPersister.saveSample(self.handle, resultGroup, result)

    def saveField(self, resultGroup, result):
        """
        Saves a FieldContainer instance to the given node. This method is
        intended for internal use only.
        resultGroup -- node at which the FieldContainer should be saved
                       in the file.
        result -- FieldContainer instance to be saved
        """
        return PyTablesPersister.saveField(self.handle, resultGroup, result)

    def saveRecipe(self, recipe, saveResults=True):
        """
        Saves a recipe

        recipe -- CompositeWorker to be saved
        saveResults -- Whether to save results of the workers
        """
        return PyTablesPersister.saveRecipe(self.handle, recipe, saveResults)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


from pyphant.core.Connectors import TYPE_IMAGE
from pyphant.wxgui2.DataVisReg import DataVisReg
import wx
from pyphant.visualizers.ConfigurablePlot import PlotPanel
import matplotlib
import scipy
import pylab
import os
DEFAULT_CMAP = 'gray'
DEFAULT_CBAD = 'red'


class ZStackPlotPanel(PlotPanel):
    def __init__(self, parent, fieldContainer):
        self.fieldContainer = fieldContainer
        self.cmap = DEFAULT_CMAP
        self.cbad = DEFAULT_CBAD
        PlotPanel.__init__(self, parent)

    def _setfc(self, fieldContainer):
        self._fieldContainer = fieldContainer
        attrs = fieldContainer.attributes
        def scale(quant, name):
            if quant is None:
                return getattr(fieldContainer.data, name[1:])()
            return quant / fieldContainer.unit
        bounds = [scale(attrs.get(name, None), name) \
                  for name in ['vmin', 'vmax']]
        self.vmin, self.vmax = bounds

    def _getfc(self):
        return self._fieldContainer
    fieldContainer = property(_getfc, _setfc)

    def getax(self, figure, img_fc, xlim=None, ylim=None):
        ax = figure.add_subplot(111)
        xdim = img_fc.dimensions[1]
        ydim = img_fc.dimensions[0]
        xmin = scipy.amin(xdim.data)
        xmax = scipy.amax(xdim.data)
        ymin = scipy.amin(ydim.data)
        ymax = scipy.amax(ydim.data)
        image0 = matplotlib.image.NonUniformImage(
            ax, extent=(xmin, xmax, ymin, ymax))
        cmap0 = getattr(pylab.cm, self.cmap)
        image0.set_cmap(cmap0)
        image0.set_norm(matplotlib.colors.Normalize(
            vmin=self.vmin, vmax=self.vmax, clip=False))
        image0.set_data(xdim.data, ydim.data, img_fc.data)
        image = matplotlib.image.NonUniformImage(
            ax, extent=(xmin, xmax, ymin, ymax))
        cmap = getattr(pylab.cm, self.cmap)
        cmap.set_bad(self.cbad, 0.7)
        image.set_cmap(cmap)
        image.set_norm(matplotlib.colors.Normalize(
            vmin=self.vmin, vmax=self.vmax, clip=False))
        image.set_data(xdim.data, ydim.data, img_fc.maskedData)
        ax.images.append(image0)
        ax.images.append(image)
        if xlim is None:
            ax.set_xlim(xmin, xmax)
        else:
            ax.set_xlim(xlim)
        if ylim is None:
            ax.set_ylim(ymin, ymax)
        else:
            ax.set_ylim(ylim)
        ax.set_xlabel(xdim.shortlabel)
        ax.set_ylabel(ydim.shortlabel)
        ax.set_title(img_fc.label)
        from pyphant.visualizers.ConfigurablePlot import F
        self.colorbar = figure.colorbar(
            image, format=F(img_fc), ax=ax)
        return ax

    def draw(self, reset=False):
        self.figure.clear()
        xlim = None
        ylim = None
        if hasattr(self, 'ax') and self.ax.get_xlim() != (0.0, 1.0) and \
               self.ax.get_ylim() != (0.0, 1.0) and not reset:
            xlim = self.ax.get_xlim()
            ylim = self.ax.get_ylim()
        self.ax = self.getax(self.figure, self.fieldContainer,
                             xlim=xlim, ylim=ylim)
        self.canvas.draw()

    def saveImage(self, img_fc, path, pixel_resolution):
        dpi = 100
        figsize = (float(pixel_resolution[0] / dpi),
                   float(pixel_resolution[1] / dpi))
        saveFigure = matplotlib.figure.Figure(figsize=figsize, dpi=dpi)
        matplotlib.backends.backend_agg.FigureCanvasBase(
            saveFigure)
        self.getax(saveFigure, img_fc)
        saveFigure.savefig(path, dpi=dpi)


class ZStackConfigPanel(wx.PyPanel):
    def __init__(self, parent, plotPanel):
        wx.PyPanel.__init__(self, parent)
        self.parent = parent
        self.plotPanel = plotPanel
        sizer1 = wx.FlexGridSizer(1, 3)
        self.prevButton = wx.Button(self, label="previous")
        sizer1.Add(self.prevButton)
        self.zChoice = wx.Choice(self, choices=self.parent.zvalues)
        self.zChoice.SetStringSelection(self.parent.zvalues[0])
        sizer1.Add(self.zChoice)
        self.nextButton = wx.Button(self, label="next")
        self.nextButton.SetDefault()
        sizer1.Add(self.nextButton)
        self.saveButton = wx.Button(self, label="save all to -->")
        sizer2 = wx.FlexGridSizer(1, 2)
        sizer2.Add(self.saveButton)
        self.dpc = wx.DirPickerCtrl(self, message="Pick a directory to save "\
                                    "all images")
        sizer2.Add(self.dpc)
        self.cmapChoice = wx.Choice(self, choices=self.parent.cmaps)
        self.cmapChoice.SetStringSelection(DEFAULT_CMAP)
        self.cmapChoice.SetToolTip(wx.ToolTip("select color map"))
        sizer3 = wx.FlexGridSizer(1, 3)
        sizer3.Add(self.cmapChoice)
        self.cbadChoice = wx.Choice(self, choices=self.parent.cbads)
        self.cbadChoice.SetStringSelection(DEFAULT_CBAD)
        self.cbadChoice.SetToolTip(wx.ToolTip("select mask color"))
        sizer3.Add(self.cbadChoice)
        self.resetButton = wx.Button(self, label="reset view")
        sizer3.Add(self.resetButton)
        sizer = wx.FlexGridSizer(3, 1)
        sizer.Add(sizer1)
        sizer.Add(sizer2)
        sizer.Add(sizer3)
        self.SetSizer(sizer)
        self.Centre()
        self.Bind(wx.EVT_BUTTON, self.onNext, self.nextButton)
        self.Bind(wx.EVT_BUTTON, self.onPrev, self.prevButton)
        self.Bind(wx.EVT_BUTTON, self.onSave, self.saveButton)
        self.Bind(wx.EVT_BUTTON, self.onReset, self.resetButton)
        self.Bind(wx.EVT_CHOICE, self.onZChanged, self.zChoice)
        self.Bind(wx.EVT_CHOICE, self.onCChanged, self.cmapChoice)
        self.Bind(wx.EVT_CHOICE, self.onCChanged, self.cbadChoice)
        self.onZChanged()

    def _getZ(self):
        return self.zChoice.GetStringSelection()

    def _setZ(self, zvalue):
        self.zChoice.SetStringSelection(zvalue)
    zvalue = property(_getZ, _setZ)

    def _getZs(self):
        return self.parent.zvalues
    zvalues = property(_getZs)

    def invalidateButtons(self):
        self.prevButton.Enable(self.zid > 0)
        self.nextButton.Enable(self.zid < len(self.zvalues) - 1)

    def onNext(self, Event):
        self.zvalue = self.zvalues[self.zid + 1]
        self.onZChanged()

    def onPrev(self, Event):
        self.zvalue = self.zvalues[self.zid - 1]
        self.onZChanged()

    def onSave(self, Event):
        directory = os.path.realpath(self.dpc.GetPath())
        dial = wx.TextEntryDialog(
            parent=self,
            message="Save all images to '%s'?\nPixel resolution:" % directory,
            caption="Are you sure?",
            defaultValue="800x600")
        retval = dial.ShowModal()
        user_text = dial.GetValue()
        dial.Destroy()
        pixel_resolution = map(int, user_text.split('x'))
        longname = self.parent.dataContainer.longname
        if retval == wx.ID_OK:
            for count, img_fc in enumerate(self.parent.dataContainer):
                path = os.path.join(directory, "%s_%03d.png" \
                                    % (longname, count))
                self.plotPanel.saveImage(img_fc, path, pixel_resolution)
            wx.MessageBox(message="Saved all images to '%s'." % directory,
                          caption='Done!')

    def onReset(self, Event):
        self.plotPanel.draw(reset=True)
        self.GetSizer().Fit(self)

    def onZChanged(self, Event=None):
        self.zid = self.zChoice.GetSelection()
        self.invalidateButtons()
        imgFC = self.parent.dataContainer[self.zid]
        self.plotPanel.fieldContainer = imgFC
        self.plotPanel.draw()
        self.GetSizer().Fit(self)

    def onCChanged(self, Event):
        self.plotPanel.cmap = self.cmapChoice.GetStringSelection()
        self.plotPanel.cbad = self.cbadChoice.GetStringSelection()
        self.plotPanel.draw()
        self.GetSizer().Fit(self)


class ZStackVisualizer(wx.Frame):
    name='Z-Stack'
    def __init__(self, dataContainer, show=True):
        self.init_zvalues(dataContainer)
        self.init_cvalues(dataContainer)
        self.dclongname = dataContainer.longname
        wx.Frame.__init__(self, None, -1, "Z-Stack " + dataContainer.longname)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        imgFC = dataContainer[0]
        self.dataContainer = dataContainer
        self.plotPanel = ZStackPlotPanel(self, imgFC)
        self.configPanel = ZStackConfigPanel(self, self.plotPanel)
        self.sizer.Add(self.configPanel, 0, wx.EXPAND)
        self.sizer.Add(self.plotPanel, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.SetSizer(self.sizer)
        self.Centre()
        self.Show(True)

    def init_zvalues(self, dataContainer):
        zunit = dataContainer.dimensions[0].unit
        self.zvalues = [str(zvalue * zunit) for zvalue \
                        in dataContainer.dimensions[0].data]

    def init_cvalues(self, dataContainer):
        self.cmaps = pylab.cm._cmapnames
        self.cbads = ['red', 'green', 'blue', 'yellow',
                      'black', 'white']
        self.cmaps.sort()
        self.cbads.sort()

    def OnExit(self, event):
        self.Close()


DataVisReg.getInstance().registerVisualizer(TYPE_IMAGE, ZStackVisualizer)



# -*- coding: utf-8 -*-
"""
Module for generating encoded strings representing data in FMF format.

 Version control ID: $Id$

Example:

        # Assumed: input encoding here is 'cp1252' (Windows ANSI), can be changed
        #          by arguments of gen_factory()

        #
        # Preparing the FMF container
        #
        factory = gen_factory(out_coding='utf-8') # we want to have utf-8 files
        fc = factory.gen_fmf()  # this is the container for the data

        #
        # Adding information to the reference section, which has been automatically
        # generated by the FMF container.
        #
        fc.add_reference_item('author', 'Michael Röttger')
        fc.add_reference_item('device', 'keyboard')

        #
        # Preparing a section and add it to the container
        #
        sec = factory.gen_section("parameters")
        sec.add_item('a','3 cm')
        sec.add_item('z',4-6j)  # adding a complex number,
                                # will be written without parantheses

        # generate a standard python datetime objects and use it as item value
        dto = datetime.datetime(2008,4,9,10,59,59)
        sec.add_item('when',dto) # will be written in ISO format

        fc.add_section(sec)

        #
        # Preparing a table and add it to the container
        # (four rows; three columns which have to be defined)
        #
        tab1 = factory.gen_table([[1,2,3],[4,5,6],[7,8,9],[10,11,12]])

        # column definition having a unit, but no dependency
        tab1.add_column_def('longname1', 's1', 'm/s')

        # column definition having a unit and one dependency
        tab1.add_column_def('longname2', 's2', 'N', ['s3'])

        # column definition having no unit and two dependencies
        tab1.add_column_def('longname3', 's3', None, ['s1','s2'])

        fc.add_table(tab1)

        #
        # Write data structure to file using the container as string
        # It will be encoded with the output encoding given above.
        #
        outfile = open('example.fmf','w')
        outfile.write(str(fc))
        outfile.close()

Please have a look at the test examples included in the source distribution
for further examples.
"""



#########################################################################################################
# TODO, Ideen:
# ------------
# Sollen wirklich Ausgabeformate bei der Tabelle erlaubt werden?
# Müssen komplexe Zahlenwerte auch in ihren einzelnen Bestandteilen formattiert werden können?
#
# Wie soll datetime-Objekten umgegangen werden? Momentan werden sie im ISO-Format mit 'T'-Trenner ausgegeben.
#
# Generell für jeden Typ ein Funktion hinterlegbar machen, die die Umwandlung in einen String
# erledigt?
#
# Das nachträgliche Hinzufügen von Zeilen ermöglichen, dabei auch mit leerer Tabelle starten können.
# Ermöglichen, einer Spaltendefinition gleich die Daten hinzuzufügen.
# Stubs sind schon vorhanden (auskommentiert).
#
# Wie soll mit Quoting umgegangen werden, z.B. \rm in shortnames? Automatisch \\ für \ ?
#########################################################################################################


import sys
import string

import datetime

import logging
_log = logging.getLogger("fmfgen")

RESERVED_MARK = '*'

LPAREN_DEPEND = '('
RPAREN_DEPEND = ')'

DEFAULT_EOL = '\r\n'

DEFAULT_OUT_ENCODING = 'utf-8' # Windows ANSI encoding, Western Europe
DEFAULT_IN_ENCODING = 'utf-8' # Windows ANSI encoding, Western Europe

DEFAULT_TABLE_LONGNAME_PATTERN = 'table %d'
DEFAULT_TABLE_SHORTNAME_PATTERN = 'T_{%d}'
DEFAULT_COL_FORMAT = '%s'
DEFAULT_COL_DELIM = '\t'

def is_unicode(s):
    return type(s)==unicode

def assure_unicode(obj, coding):
    """Return a unicode string for the given object.

     If the obj is a string, the given coding will be used for conversion.
     Complex numbers will be written without parantheses.
     Datetime objects will be printed in isoformat (including 'T')."""
    if isinstance(obj,complex):
        obj = str(obj)[1:-1]
    elif isinstance(obj,datetime.datetime):
        obj = obj.isoformat()
    if is_unicode(obj):
        return obj
    else:
        return unicode(obj,coding)

class _xproperty(property):
    """Extension of the built-in properties.
    Automatically creates a getter and optional a setter, if no
    reference to a method is given, but  a name of a variable instead.
    """
    def __init__(self, fget=None, fset=None, fdel=None, doc=None, *args, **kwds):
        """Constructor with automatically generated getter and setter.
        The following constructor automatically creates a getter and a setter,
        if given as variable name. This reduces stupid method declarations.

        See also:
            http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/110061
        """
        super(_xproperty, self).__init__(
          (isinstance(fget, str) and (lambda obj: getattr(obj, fget)) or fget),
          (isinstance(fset, str) and (lambda obj, val: setattr(obj, fset, val)) or fset),
          fdel, doc)
        # print "doc=",doc, " self.__doc__", self.__doc__
        #if doc is not None:
        #    if self.__doc__ is None:
        #        self.__doc__=""
        #    self.__doc__+=doc
        self.__doc__=doc

class _AutoProperty(_xproperty):
    """Extension of the built-in properties.
    Automatically creates a getter and optional a setter, if no
    reference to a method is given, but  a name of a variable instead.
    """
    def __init__(self, fget=None, fset=None, fdel=None, doc=None,*args,**kwds):
        """Constructor with automatically generated getter and setter.
        The following constructor automatically creates a getter and a setter,
        if given as variable name. This reduces stupid method declarations.

        See also:
            http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/110061
        """
        # if both fget and fset are strings, they must be equal
        assert (not isinstance(fget, str)) or (not isinstance(fset, str)) or (fget==fset),\
               "fget and fset should be equal, if both strings"

        super(_AutoProperty,self).__init__(fget,fset,fdel,doc,*args,**kwds)
        self.__doc__=doc


class FMFBuildException(Exception):
    pass

class _FMFElement(object):
    def __init__(self, in_coding=DEFAULT_IN_ENCODING,
                 out_coding=DEFAULT_OUT_ENCODING, eol=DEFAULT_EOL, *args, **kwds):
        """Initialise general element of FMF structure with common arguments."""

        # check, if given coding is known
        #try:
        #    s = u"jklö"
        #    s = s.encode(coding) # unicode -> given coding
        #except LookupError, e:
        #    FMFBuildException("FMF builder initialized with unknown encoding: %s" % (coding,))

        self._in_coding = in_coding
        self._out_coding = out_coding
        self._eol = eol
        self._factory = kwds['factory']

    def __str__(self):
        return self.unicode().encode(self._out_coding)

class _Item(_FMFElement):

    def __init__(self, metatag, value, *args, **kwds):
        """Initialise item to be inserted in a section.

         metatag -- key of the item
         value   -- value of the item

        Both arguments are assumed to be unicode.
        """
        super(_Item,self).__init__(*args,**kwds)
        assert is_unicode(metatag), "Given metatag is not unicode: %s" % (metatag,)
        assert is_unicode(value), "Given value is not unicode: %s" % (value,)

        # metatag should be alphanumerical?! What does this mean?
        self._metatag = metatag
        self._value = value

    def unicode(self):
        return u"%s: %s%s" % (self._metatag, self._value, self._eol)

    metatag = _AutoProperty('_metatag')
    value = _AutoProperty('_value')


class _ColumnDef(_Item):
    def __init__(self, longname, shortname, unit=None,
                 deps=[], error = None,format=DEFAULT_COL_FORMAT, *args, **kwds):
        """Column definition as item for a data definition section.

         longname  --  long name for the column, human-readable
         shortname --  typically a symbol
         unit      --  unit like 'm/s' (optional, default: None)
         deps      --  list of dependencies, which are themselves short names
                       of other columns (optional, default: [])
         error     --  Reference to the column symbol listing the respective error
         format    --  format specifier used for column values (optional)

        The instances given for 'longname', 'shortname', 'deps' items and 'unit'
        are assumed to be unicode.
        """

        value = shortname
        if len(deps)>0:
            value += LPAREN_DEPEND
            for d in deps[:-1]:
                value += u"%s," % (d,)
            value += deps[-1]+RPAREN_DEPEND
        if unit is not None:
            value += u" [%s]" % (unit,)
        if error is not None:
            value += u" +- %s" % (error,)
        super(_ColumnDef,self).__init__( longname, value, *args, **kwds)

        self._shortname = shortname
        self._unit = unit
        self._deps = deps
        self._error = error
        self._format = format

    longname = _AutoProperty('_metatag')
    shortname = _AutoProperty('_shortname')
    unit = _AutoProperty('_unit')
    deps = _AutoProperty('_deps')
    error = _AutoProperty('_error')
    format = _AutoProperty('_format')

class _Section(_FMFElement):

    def __init__(self, tag, reserved=False, *args, **kwds):
        """Initialise a section.

         tag        -- name of the section
         reserved   -- is this a reserved tag? (default:False)

         The 'tag' is assumed to be a unicode string.
        """
        super(_Section,self).__init__(*args,**kwds)
        _log.debug("Generating an FMF section tagged '%s'. Reserved section? %s", tag, reserved)
        self._tag = tag
        self._reserved = reserved
        self._items = []

    def __len__(self):
        return len(self._items)

    def _header(self):
        if self._reserved:
            c = RESERVED_MARK
        else:
            c = ''
        return u"[%s%s]%s" % (c,self._tag,self._eol)

    def unicode(self):
        s = self._header()
        s += u''.join([ it.unicode() for it in self._items])
        return s

    def add_item(self, metatag, value, arg_coding=None):
        """Adding a key (aka 'metatag') / value pair as item to this section.

          metatag    --  key of the item
          value      --  value of the item
          arg_coding --  input encoding of 'metatag' and 'value'.
                         Optional, default is to use the factory's settings.
        """
        item = self._factory._gen_item(metatag, value,arg_coding=arg_coding)
        self._items.append(item)

    def add_quantity_item(self, longname, value, shortname=None, unit=None,
                          error_value=None, error_unit=None, rel_error=False, arg_coding=None):
        """Adding a quantity as item to this section.

          longname    --  used as key/metatag of this item
          value       --  numeric value of the quantity
          shortname   --  optional, short name of the item
          unit        --  optional, unit of the value
          error_value --  optional, numeric error value, interpretation depends on
                          'rel_error'
          error_unit  --  optional, unit of the error value; only useful for absolute
                          errors; if not given, the unit of the quantity's value
                          will be taken
          rel_error   --  optional, boolean, defaults to 'False'; if
                          'True', error_value is assumed to be a relative error, so
                          the absolute error is

                            +/- value*error_value

          arg_coding --  input encoding of the arguments.
                         Optional, default is to use the factory's settings.

         It is not checked whether the unit and the error's unit are compatible.
        """
        if (error_unit is not None) and rel_error:
            raise FMFBuildException("Specification of error's unit not useful when giving a relative error value.")
        if (error_unit is not None) and (unit is None):
            raise FMFBuildException("Specification of error's unit must go along with specification of value's unit.")


        if error_value is None:
            combined_value = str(value)
            if unit is not None:
                combined_value += " %s" % (unit,)
        else:
            if error_unit is None:
                if rel_error:
                    combined_value = "(1 \\pm %s) %s" % (error_value,value)
                else:
                    combined_value = "(%s \\pm %s)" % (value,error_value)
                if unit is not None:
                    combined_value += " %s" % (unit,)
            else:
                combined_value = "%s %s \\pm %s %s" % (value,unit,error_value,error_unit)

        if shortname is not None:
            combined_value = "%s=%s" % (shortname,combined_value)

        self.add_item(longname, combined_value, arg_coding=arg_coding)


    tag = _AutoProperty('_tag')
    reserved = _AutoProperty('_reserved')

class _Table(_FMFElement):
    def __init__(self, seq2D, longname, shortname, autonames=False, *args, **kwds):
        """Initialise a table.

         seq2D     -- sequence of sequences allowing access by

                       seq2D[index1][index2]

         longname  -- long, human readable name for the table
         shortname -- symbol for the table
         autonames -- flag indicating wheter both names are automatically
                      generated and not explicitly chosen by a human user (default:False)

         'longname' and 'shortname' are assumed to be unicode strings.
         The tbale is able generate its own data definition section and data section.
        """
        super(_Table,self).__init__(*args,**kwds)

        self._seq2D = seq2D

        self._check_row_lengths(seq2D)

        if not is_unicode(longname):
                raise FMFBuildException("Table must be initialized with a unicode string as longname.")
        if not is_unicode(shortname):
                raise FMFBuildException("Table must be initialized with a unicode string  as shortname.")

        self._longname = longname
        self._shortname = shortname
        self._autonames = autonames

        self._coldefs = []

        self._suppress_section_shortnames = False

    def _check_row_lengths(self, seq2D):
        """Check, if all rows have an equal numbers of elements.

        Raise an exception, if not.
        """
        len0 = len(seq2D[0])
        for row in seq2D:
            if len(row)!=len0:
                raise FMFBuildException("Table '%s' has rows with different lengths." \
                                    % (self._shortname,))

    def add_column_def(self, longname, shortname, unit=None, dependencies=[],
                       error = None,
                       format=DEFAULT_COL_FORMAT, arg_coding=None): # , data=None):
        """Add a column definition to the table.

         longname  --  long name for the column, human-readable
         shortname --  typically a symbol
         unit      --  unit like 'm/s' (optional, default: None)
         deps      --  list of dependencies, which are themselves short names
                       of other columns (optional, default: [])
         format    --  format specifier used for column values (optional)
         arg_coding--  input encoding of 'longname', 'shortname', 'unit' and
                       elements of 'dependencies'. Optional, default is to use
                       the factory's settings.
        """
        # Proposal for an additional argument:
        #         data      --  sequence with references to data items; length must be
        #                       equal to current number of rows in the table; data
        #                       is joined to the current table (optional)
        coldef = self._factory._gen_column_def(longname, shortname, unit, dependencies,error,
                                               format, arg_coding)
        self._coldefs.append(coldef)
        #if not data is None:
        #    if len(data)!=self.num_rows:
        #        raise FMFBuildException("Table '%s' has %d rows so far, " % (self._shortname,self.num_rows)+\
        #                                "cannot add column with %d rows." % (len(data),))
        #    # add column
        #    pass

    #def add_row(self, data):
    #    """Add a row to the existing table.
    #
    #      data -- sequence having the same number of items then the
    #              table has column definitions
    #    """
    #    pass

    def unicode(self):
        #
        # check for number of data definitions
        #
        if len(self._coldefs)!=len(self._seq2D[0]):
            raise FMFBuildException("Table '%s' has %d columns, but %d columns definitions." \
                                    % (self._shortname,len(self._seq2D[0]),len(self._coldefs)))
        #
        # check, whether dependencies are ok
        #
        shortnames = [ cd.shortname for cd in self._coldefs]
        for cd in self._coldefs:
            for dep in cd.deps:
                if not dep in shortnames:
                    raise FMFBuildException("Shortname '%s', which has been given as dependency " % (dep,)\
                                            +"for column '%s', is not defined in a separate column."\
                                            % (cd.longname,))
        #
        # prepare data definition section
        #
        datadef_section_name = u'*data definitions'
        if not self._suppress_section_shortnames:
            datadef_section_name += u": %s" % (self._shortname,)
        datadef_section = self._factory.gen_section(datadef_section_name)
        for cd in self._coldefs:
            datadef_section.add_item(cd.metatag, cd.value)

        #
        # prepare data section
        #
        data_section_name = u'*data'
        if not self._suppress_section_shortnames:
            data_section_name += u": %s" % (self._shortname,)
        data_section = self._factory.gen_section(data_section_name)
        data_lines = []
        col_formats = [ cd.format for cd in self._coldefs]
        for i,row in enumerate(self._seq2D):
            line = u""
            for j,value in enumerate(row):
                if isinstance(value,complex):
                    value = str(value)[1:-1] # leave out parantheses
                elif isinstance(value,datetime.datetime):
                    value = value.isoformat()
                if isinstance(value,str):
                    value = unicode(value,self._in_coding)

                try:
                    line += col_formats[j] % (value,)
                except:
                    raise FMFBuildException("Couldn't format '%s' with %s" % (value,col_formats[j]))
                if j<len(row)-1:
                    line += DEFAULT_COL_DELIM
                else:
                    line += self._eol
            data_lines.append(line)


        #
        # construct text representation of this table
        #
        s = datadef_section.unicode()
        s += data_section.unicode()
        for dl in data_lines:
            s += dl
        return s

    def _get_num_rows(self):
        """Return the number of rows."""
        return len(self._seq2D)

    def _get_num_cols(self):
        """Return the number of columns."""
        if len(self._seq2D)>0:
            return len(self._seq2D[0])
        else:
            return 0

    shortname = _AutoProperty('_shortname')
    longname = _AutoProperty('_longname')
    autonames = _AutoProperty('_autonames')
    suppress_section_shortnames = _AutoProperty('_suppress_section_shortnames',\
                                                '_suppress_section_shortnames')
    num_rows = _AutoProperty(_get_num_rows)
    num_cols = _AutoProperty(_get_num_cols)

class _FMFContainer(_FMFElement):

    def __init__(self, *args, **kwds):
        """Initialise a container holding data by means of FMF data structure."""
        super(_FMFContainer,self).__init__(*args,**kwds)

        self._user_sections = {}
        self._user_sections_tags = [] # for saving the order
        self._reference_section = _Section('reference', reserved=True, *args, **kwds)

        self._tables = []
        self._tables_section = None

    def add_reference_item(self, metatag, value, arg_coding=None):
        """Add an item for the reference section.

         metatag   -- key of the reference item, e.g. 'author'
         value     -- value of the reference item
         arg_coding -- if strings are encoded differently from default encoding,
                      it can be specified here (default: encoding of the container)"""
        if arg_coding is None:
            arg_coding = self._in_coding
        self._reference_section.add_item(metatag, value, arg_coding=arg_coding)

    def add_section(self, section):
        """Add a section.

          section -- the section to be added
        """

        if section.reserved:
            raise FMFBuildException("User defined sections shouldn't marked as reserved (section: %s)." % (section.tag,))

        if section.tag in self._user_sections.keys():
            raise FMFBuildException("Section '%s' is already included. Each section can only be added once.")

        self._user_sections[section.tag] = section
        self._user_sections_tags.append(section.tag)

    def add_table(self, table):
        """Add a table.

          table -- the table to be added
        """
        self._tables.append(table)
        if ((not table.autonames) or (len(self._tables)>1)) \
               and self._tables_section is None:
            self._tables_section = self._factory.gen_section('*tables')
            if len(self._tables)>1:
                # this call adds the second table, so hand in former table
                tab0 = self._tables[0]
                self._tables_section.add_item(tab0.longname,tab0.shortname)
        if self._tables_section is not None:
            self._tables_section.add_item(table.longname,table.shortname)

    def unicode(self):

        if len(self._tables)==0:
            raise FMFBuildException("Cannot generate FMF file content to which no table has been added.")

        # *reference
        # user defined sections
        # *data definitions
        # *data

        s = u"; -*- coding: %s -*-%s" % (self._out_coding,self._eol)
        s += self._reference_section.unicode()
        for tag in self._user_sections_tags:
            s += self._user_sections[tag].unicode()
        if not self._tables_section is None:
            s += self._tables_section.unicode()
        for tab in self._tables:
            tab.suppress_section_shortnames = (self._tables_section is None)
            s += tab.unicode()
        # s += self._eol
        return s

    sections = _AutoProperty('_sections')

class _FMFElementFactory(object):
    """Produces elements which can be inserted into an FMF data structure."""

    def __init__(self,
                 in_coding=DEFAULT_IN_ENCODING,
                 out_coding=DEFAULT_OUT_ENCODING,
                 eol=DEFAULT_EOL):
        self._in_coding = in_coding
        self._out_coding = out_coding
        self._eol = eol

        self._table_count = 0

        self._element_kwds = { 'in_coding': in_coding,
                               'out_coding':out_coding,
                               'eol': eol,
                               'factory': self}

    def _gen_item(self, metatag, value, arg_coding=None, *args, **kwds):
        """Generate an item for latter insertion into a section.

         metatag    -- key of the item
         value      -- value of the item
         arg_coding -- if encoding of 'metatag' and 'value' differs
                       from standard input encoding, it can be specified here
                        (Default: use input encoding given when instanitating factory)

         Usually not directly called. Use section methods instead.
        """
        if arg_coding is None:
            arg_coding = self._in_coding
        metatag = assure_unicode(metatag, arg_coding)
        value = assure_unicode(value, arg_coding)
        kwds.update(self._element_kwds) # mix in the coding and line marker information
        return _Item(metatag, value, *args, **kwds)

    def _gen_column_def(self, longname, shortname, unit=None, dependencies=[], error = None,
                       format=DEFAULT_COL_FORMAT, arg_coding=None, *args, **kwds):
        """Generate a column definition.

         longname  --  long name for the column, human-readable
         shortname --  typically a symbol
         unit      --  unit like 'm/s' (optional, default: None)
         deps      --  list of dependencies, which are themselves short names
                       of other columns (optional, default: [])
         format    --  format specifier used for column values (optional)
         arg_coding--  input encoding of 'longname', 'shortname', 'unit' and
                       elements of 'dependencies'. Optional, default is to use
                       the factory's settings.

         Usually not directly called. Use table methods instead.
        """
        if arg_coding is None:
            arg_coding = self._in_coding
        longname = assure_unicode(longname, arg_coding)
        shortname = assure_unicode(shortname, arg_coding)
        if (unit is not None) and str(unit)!='1':
            unit = assure_unicode(unit, arg_coding)
            if unit.startswith('1.0 '):
                unit=unit[4:]
        else:
            unit = None
        for i,d in enumerate(dependencies):
            dependencies[i] = assure_unicode(d, arg_coding)
        kwds.update(self._element_kwds) # mix in the coding and line marker information
        return _ColumnDef(longname, shortname, unit, dependencies, error, format, *args, **kwds)

    def gen_section(self, tag, arg_coding=None, *args, **kwds):
        """Generate a section for latter inclusion in a FMF file.

          tag        -- tag of the section
          arg_coding -- if encoding of 'tag' differs from standard input encoding,
                        it can be specified here
                        (Default: use input encoding given when instanitating factory)
        """
        if arg_coding is None:
            arg_coding = self._in_coding
        tag = assure_unicode(tag, arg_coding)
        reserved = tag.startswith(RESERVED_MARK)
        if reserved:
            tag = tag[len(RESERVED_MARK):]
        kwds.update(self._element_kwds) # mix in the coding and line marker information
        return _Section(tag, reserved=reserved, *args, **kwds)

    def gen_table(self, seq2D, longname=None, shortname=None, arg_coding=None, *args, **kwds):
        """Return a table for inclusion into FMF container.

         seq2D      -- sequence of sequences representing the table
         shortname  -- short name used to reference table definitions (optional)
         longname   -- long name used to describe the table (optional)
         arg_coding -- if encoding of 'longname' and 'shortname' differs from
                       standard input encoding, it should be specified here
                       (Default: use input encoding given when instantiating factory)

         If shortname or longname is not given, a default name will be constructed
         using a global counter for tables as index. If both names are not given,
         the default names will be built with the same index number.
        """
        if arg_coding is None:
            arg_coding = self._in_coding

        autonames = (longname is None) and (shortname is None)
        if (longname is None) or (shortname is None):
            self._table_count += 1
            if longname is None:
                longname = DEFAULT_TABLE_LONGNAME_PATTERN % (self._table_count,)
            if shortname is None:
                shortname = DEFAULT_TABLE_SHORTNAME_PATTERN % (self._table_count,)

        longname = assure_unicode(longname, arg_coding)
        shortname = assure_unicode(shortname, arg_coding)
        kwds.update(self._element_kwds) # mix in the coding and line marker information
        return _Table( seq2D, longname, shortname, autonames, *args, **kwds)

    def gen_fmf(self, *args, **kwds):
        """Generate a fmf structure suitable for output in a file.
        """
        kwds.update(self._element_kwds) # mix in the coding and line marker information
        return _FMFContainer(*args, **kwds)


def gen_factory( in_coding=DEFAULT_IN_ENCODING,
                 out_coding=DEFAULT_OUT_ENCODING,
                 eol=DEFAULT_EOL):
    """Return a factory for elements of a FMF data structure using common settings.

     in_coding  --  default encoding assumed for incoming strings,
                    when not explicitly given (default: cp1252)
     out_coding --  default encoding assumed for output string,
                    when not explicitly given (default: cp1252)
     eol        --  end of line marker (default: Carriage Return + Line Feed)

     The default arguments are typical for Windows systems.
    """
    return _FMFElementFactory(in_coding,out_coding,eol)








# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
"""


from matplotlib.image import AxesImage
import numpy as np
from numpy import ma
from matplotlib import _image, colors as mcolors

class NonUniformImage(AxesImage):
    def __init__(self, ax,
                 **kwargs
                ):
        AxesImage.__init__(self, ax,
                           **kwargs)

    def make_image(self, magnification=1.0):
        if self._A is None:
            raise RuntimeError('You must first set the image array')

        x0, y0, v_width, v_height = self.axes.viewLim.bounds
        l, b, r, t = self.axes.bbox.extents
        width = (round(r) + 0.5) - (round(l) - 0.5)
        height = (round(t) + 0.5) - (round(b) - 0.5)
        width *= magnification
        height *= magnification
        im = _image.pcolor(self._Ax, self._Ay, self._A,
                           height, width,
                           (x0, x0+v_width, y0, y0+v_height))
        fc = self.axes.get_frame().get_facecolor()
        bg = mcolors.colorConverter.to_rgba(fc, 0)
        im.set_bg(*bg)
        return im

    def set_data(self, x, y, A):
        x = np.asarray(x,np.float32)
        y = np.asarray(y,np.float32)
        A = ma.asarray(A)
        if len(x.shape) != 1 or len(y.shape) != 1\
           or A.shape[0:2] != (y.shape[0], x.shape[0]):
            raise TypeError("Axes don't match array shape")
        if len(A.shape) not in [2, 3]:
            raise TypeError("Can only plot 2D or 3D data")
        if len(A.shape) == 3 and A.shape[2] not in [1, 3, 4]:
            raise TypeError("3D arrays must have three (RGB) or four (RGBA) color components")
        if len(A.shape) == 3 and A.shape[2] == 1:
            A.shape = A.shape[0:2]
        if len(A.shape) == 2:
            if A.dtype != np.uint8:
                A = (self.cmap(self.norm(A))*255).astype(np.uint8)
            else:
                A = np.repeat(A[:,:,np.newaxis], 4, 2)
                A[:,:,3] = 255
        else:
            if A.dtype != np.uint8:
                A = (255*A).astype(np.uint8)
            if A.shape[2] == 3:
                B = zeros(tuple(list(A.shape[0:2]) + [4]), np.uint8)
                B[:,:,0:3] = A
                B[:,:,3] = 255
                A = B
        self._A = A
        self._Ax = x
        self._Ay = y
        self._imcache = None

    def set_array(self, *args):
        raise NotImplementedError('Method not supported')

    def set_interpolation(self, s):
        if s != None and s != 'nearest':
            raise NotImplementedError('Only nearest neighbor supported')
        AxesImage.set_interpolation(self, s)

    def get_extent(self):
        if self._A is None:
            raise RuntimeError('Must set data first')
        return self._Ax[0], self._Ax[-1], self._Ay[0], self._Ay[-1]

    def set_filternorm(self, s):
        pass

    def set_filterrad(self, s):
        pass

    def set_norm(self, norm):
        if self._A is not None:
            raise RuntimeError('Cannot change colors after loading data')
        cm.ScalarMappable.set_norm(self, norm)

    def set_cmap(self, cmap):
        if self._A is not None:
            raise RuntimeError('Cannot change colors after loading data')
        cm.ScalarMappable.set_cmap(self, norm)




# -*- coding: utf-8 -*-
"""
Bottle is a fast and simple micro-framework for small web applications. It
offers request dispatching (Routes) with url parameter support, templates,
key/value databases, a built-in HTTP Server and adapters for many third party
WSGI/HTTP-server and template engines - all in a single file and with no
dependencies other than the Python Standard Library.

Homepage and documentation: http://wiki.github.com/defnull/bottle

Special thanks to Stefan Matthias Aust [http://github.com/sma]
  for his contribution to SimpleTemplate

Licence (MIT)
-------------

    Copyright (c) 2009, Marcel Hellkamp.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.


Example
-------

    from bottle import route, run, request, response, send_file, abort

    @route('/')
    def hello_world():
        return 'Hello World!'

    @route('/hello/:name')
    def hello_name(name):
        return 'Hello %s!' % name

    @route('/hello', method='POST')
    def hello_post():
        name = request.POST['name']
        return 'Hello %s!' % name

    @route('/static/:filename#.*#')
    def static_file(filename):
        send_file(filename, root='/path/to/static/files/')

    run(host='localhost', port=8080)

"""

__author__ = 'Marcel Hellkamp'
__version__ = '0.6.3'
__license__ = 'MIT'

import sys
import cgi
import mimetypes
import os
import os.path
import traceback
import re
import random
import threading
import time
import email.utils
from wsgiref.headers import Headers as HeaderWrapper
from Cookie import SimpleCookie
import anydbm as dbm
import subprocess
import thread


try:
    from urlparse import parse_qs
except ImportError:
    from cgi import parse_qs

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

try:
    try:
        from json import dumps as json_dumps
    except ImportError:
        from simplejson import dumps as json_dumps
except ImportError:
    json_dumps = None






# Exceptions and Events

class BottleException(Exception):
    """ A base class for exceptions used by bottle. """
    pass


class HTTPError(BottleException):
    """
    A way to break the execution and instantly jump to an error handler.
    """
    def __init__(self, status, text):
        self.output = text
        self.http_status = int(status)
        BottleException.__init__(self, status, text)

    def __repr__(self):
        return 'HTTPError(%d,%s)' % (self.http_status, repr(self.output))

    def __str__(self):
        return HTTP_ERROR_TEMPLATE % {
            'status' : self.http_status,
            'url' : request.path,
            'error_name' : HTTP_CODES.get(self.http_status, 'Unknown').title(),
            'error_message' : ''.join(self.output)
        }


class BreakTheBottle(BottleException):
    """
    Not an exception, but a straight jump out of the controller code.
    Causes the Bottle to instantly call start_response() and return the
    content of output
    """
    def __init__(self, output):
        self.output = output






# WSGI abstraction: Request and response management

_default_app = None
def default_app(newapp = None):
    """
    Returns the current default app or sets a new one.
    Defaults to an instance of Bottle
    """
    global _default_app
    if newapp:
        _default_app = newapp
    if not _default_app:
        _default_app = Bottle()
    return _default_app


class Bottle(object):

    def __init__(self, catchall=True, optimize=False, autojson=True):
        self.simple_routes = {}
        self.regexp_routes = {}
        self.default_route = None
        self.error_handler = {}
        self.optimize = optimize
        self.autojson = autojson
        self.catchall = catchall
        self.serve = True

    def match_url(self, url, method='GET'):
        """
        Returns the first matching handler and a parameter dict or (None, None)
        """
        url = url.strip().lstrip("/ ")
        # Search for static routes first
        route = self.simple_routes.get(method,{}).get(url,None)
        if route:
            return (route, {})

        routes = self.regexp_routes.get(method,[])
        for i in range(len(routes)):
            match = routes[i][0].match(url)
            if match:
                handler = routes[i][1]
                if i > 0 and self.optimize and random.random() <= 0.001:
                    routes[i-1], routes[i] = routes[i], routes[i-1]
                return (handler, match.groupdict())
        if self.default_route:
            return (self.default_route, {})
        if method == 'HEAD': # Fall back to GET
            return self.match_url(url)
        else:
            return (None, None)

    def add_controller(self, route, controller, **kargs):
        """ Adds a controller class or object """
        if '{action}' not in route and 'action' not in kargs:
            raise BottleException("Routes to controller classes or object MUST"
                " contain an {action} placeholder or use the action-parameter")
        for action in (m for m in dir(controller) if not m.startswith('_')):
            handler = getattr(controller, action)
            if callable(handler) and action == kargs.get('action', action):
                self.add_route(route.replace('{action}', action), handler, **kargs)

    def add_route(self, route, handler, method='GET', simple=False, **kargs):
        """ Adds a new route to the route mappings. """
        if isinstance(handler, type) and issubclass(handler, BaseController):
            handler = handler()
        if isinstance(handler, BaseController):
            self.add_controller(route, handler, method=method, simple=simple, **kargs)
            return
        method = method.strip().upper()
        route = route.strip().lstrip('$^/ ').rstrip('$^ ')
        if re.match(r'^(\w+/)*\w*$', route) or simple:
            self.simple_routes.setdefault(method, {})[route] = handler
        else:
            route = re.sub(r':([a-zA-Z_]+)(?P<uniq>[^\w/])(?P<re>.+?)(?P=uniq)',
                           r'(?P<\1>\g<re>)',route)
            route = re.sub(r':([a-zA-Z_]+)', r'(?P<\1>[^/]+)', route)
            route = re.compile('^%s$' % route)
            self.regexp_routes.setdefault(method, []).append([route, handler])

    def route(self, url, **kargs):
        """
        Decorator for request handler.
        Same as add_route(url, handler, **kargs).
        """
        def wrapper(handler):
            self.add_route(url, handler, **kargs)
            return handler
        return wrapper

    def set_default(self, handler):
        self.default_route = handler

    def default(self):
        """ Decorator for request handler. Same as add_defroute( handler )."""
        def wrapper(handler):
            self.set_default(handler)
            return handler
        return wrapper

    def set_error_handler(self, code, handler):
        """ Adds a new error handler. """
        self.error_handler[int(code)] = handler

    def error(self, code=500):
        """
        Decorator for error handler.
        Same as set_error_handler(code, handler).
        """
        def wrapper(handler):
            self.set_error_handler(code, handler)
            return handler
        return wrapper

    def __call__(self, environ, start_response):
        """ The bottle WSGI-interface. """
        request.bind(environ)
        response.bind()
        try: # Unhandled Exceptions
            try: # Bottle Error Handling
                if not self.serve:
                    abort(503, "Server stopped")
                handler, args = self.match_url(request.path, request.method)
                if not handler:
                    raise HTTPError(404, "Not found")
                output = handler(**args)
                db.close()
            except BreakTheBottle, e:
                output = e.output
            except HTTPError, e:
                response.status = e.http_status
                output = self.error_handler.get(response.status, str)(e)
            # output casting
            if request.method.upper() == 'HEAD':
                output = []
            elif hasattr(output, 'read'):
                output = environ.get('wsgi.file_wrapper',
                  lambda x: iter(lambda: x.read(8192), ''))(output)
            elif self.autojson and json_dumps and isinstance(output, dict):
                output = json_dumps(output)
                response.content_type = 'application/json'
            if isinstance(output, unicode):
                response.header['Content-Length'] = str(len(output))
                output = [output.encode(response.charset)]
            if isinstance(output, str):
                response.header['Content-Length'] = str(len(output))
                output = [output]
            if not hasattr(output, '__iter__'):
                raise TypeError('Request handler for route "%s" returned [%s] '
                'which is not iterable.' % (request.path, type(output).__name__))
        except (KeyboardInterrupt, SystemExit, MemoryError):
            raise
        except Exception, e:
            response.status = 500
            if self.catchall:
                err = "Unhandled Exception: %s\n" % (repr(e))
                if DEBUG:
                    err += TRACEBACK_TEMPLATE % traceback.format_exc(10)
                output = [str(HTTPError(500, err))]
                request._environ['wsgi.errors'].write(err)
            else:
                raise
        status = '%d %s' % (response.status, HTTP_CODES[response.status])
        start_response(status, response.wsgiheaders())
        return output



class Request(threading.local):
    """ Represents a single request using thread-local namespace. """

    def bind(self, environ):
        """
        Binds the enviroment of the current request to this request handler
        """
        self._environ = environ
        self.environ = self._environ
        self._GET = None
        self._POST = None
        self._GETPOST = None
        self._COOKIES = None
        self.path = self._environ.get('PATH_INFO', '/').strip()
        if not self.path.startswith('/'):
            self.path = '/' + self.path

    @property
    def method(self):
        """ Get the request method (GET,POST,PUT,DELETE,...) """
        return self._environ.get('REQUEST_METHOD', 'GET').upper()

    @property
    def query_string(self):
        """ Get content of QUERY_STRING """
        return self._environ.get('QUERY_STRING', '')

    @property
    def input_length(self):
        """ Get content of CONTENT_LENGTH """
        try:
            return int(self._environ.get('CONTENT_LENGTH', '0'))
        except ValueError:
            return 0

    @property
    def GET(self):
        """ Get a dict with GET parameters. """
        if self._GET is None:
            data = parse_qs(self.query_string, keep_blank_values=True)
            self._GET = {}
            for key, value in data.iteritems():
                if len(value) == 1:
                    self._GET[key] = value[0]
                else:
                    self._GET[key] = value
        return self._GET

    @property
    def POST(self):
        """ Get a dict with parsed POST or PUT data. """
        if self._POST is None:
            data = cgi.FieldStorage(fp=self._environ['wsgi.input'],
                environ=self._environ, keep_blank_values=True)
            self._POST  = {}
            for item in data.list:
                name = item.name
                if not item.filename:
                    item = item.value
                self._POST.setdefault(name, []).append(item)
            for key in self._POST:
                if len(self._POST[key]) == 1:
                    self._POST[key] = self._POST[key][0]
        return self._POST

    @property
    def params(self):
        """ Returns a mix of GET and POST data. POST overwrites GET """
        if self._GETPOST is None:
            self._GETPOST = dict(self.GET)
            self._GETPOST.update(dict(self.POST))
        return self._GETPOST

    @property
    def COOKIES(self):
        """ Returns a dict with COOKIES. """
        if self._COOKIES is None:
            raw_dict = SimpleCookie(self._environ.get('HTTP_COOKIE',''))
            self._COOKIES = {}
            for cookie in raw_dict.itervalues():
                self._COOKIES[cookie.key] = cookie.value
        return self._COOKIES


class Response(threading.local):
    """ Represents a single response using thread-local namespace. """

    def bind(self):
        """ Clears old data and creates a brand new Response object """
        self._COOKIES = None
        self.status = 200
        self.header_list = []
        self.header = HeaderWrapper(self.header_list)
        self.content_type = 'text/html'
        self.error = None
        self.charset = None

    def wsgiheaders(self):
        ''' Returns a wsgi conform list of header/value pairs '''
        for c in self.COOKIES.itervalues():
            self.header.add_header('Set-Cookie', c.OutputString())
        return [(h.title(), str(v)) for h, v in self.header.items()]

    @property
    def COOKIES(self):
        if not self._COOKIES:
            self._COOKIES = SimpleCookie()
        return self._COOKIES

    def set_cookie(self, key, value, **kargs):
        """
        Sets a Cookie. Optional settings:
        expires, path, comment, domain, max-age, secure, version, httponly
        """
        self.COOKIES[key] = value
        for k, v in kargs.iteritems():
            self.COOKIES[key][k] = v

    def get_content_type(self):
        """ Get the current 'Content-Type' header. """
        return self.header['Content-Type']

    def set_content_type(self, value):
        if 'charset=' in value:
            self.charset = value.split('charset=')[-1].split(';')[0].strip()
        self.header['Content-Type'] = value

    content_type = property(get_content_type, set_content_type, None,
                            get_content_type.__doc__)


class BaseController(object):
    _singleton = None
    def __new__(cls, *a, **k):
        if not cls._singleton:
            cls._singleton = object.__new__(cls, *a, **k)
        return cls._singleton


def abort(code=500, text='Unknown Error: Appliction stopped.'):
    """ Aborts execution and causes a HTTP error. """
    raise HTTPError(code, text)


def redirect(url, code=307):
    """ Aborts execution and causes a 307 redirect """
    response.status = code
    response.header['Location'] = url
    raise BreakTheBottle("")


def send_file(filename, root, guessmime = True, mimetype = None):
    """ Aborts execution and sends a static files as response. """
    root = os.path.abspath(root)
    filename = os.path.abspath(os.path.join(root, filename))

    if not filename.startswith(root):
        abort(401, "Access denied.")
    if not os.path.exists(filename) or not os.path.isfile(filename):
        abort(404, "File does not exist.")
    if not os.access(filename, os.R_OK):
        abort(401, "You do not have permission to access this file.")

    if guessmime and not mimetype:
        mimetype = mimetypes.guess_type(filename)[0]
    if not mimetype: mimetype = 'text/plain'
    response.content_type = mimetype

    stats = os.stat(filename)
    if 'Last-Modified' not in response.header:
        lm = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(stats.st_mtime))
        response.header['Last-Modified'] = lm
    if 'HTTP_IF_MODIFIED_SINCE' in request.environ:
        ims = request.environ['HTTP_IF_MODIFIED_SINCE']
        # IE sends "<date>; length=146"
        ims = ims.split(";")[0].strip()
        ims = parse_date(ims)
        if ims is not None and ims >= stats.st_mtime:
           abort(304, "Not modified")
    if 'Content-Length' not in response.header:
        response.header['Content-Length'] = str(stats.st_size)
    raise BreakTheBottle(open(filename, 'rb'))


def parse_date(ims):
    """
    Parses date strings usually found in HTTP header and returns UTC epoch.
    Understands rfc1123, rfc850 and asctime.
    """
    try:
        ts = email.utils.parsedate_tz(ims)
        if ts is not None:
            if ts[9] is None:
                return time.mktime(ts[:8] + (0,)) - time.timezone
            else:
                return time.mktime(ts[:8] + (0,)) - ts[9] - time.timezone
    except (ValueError, IndexError):
        return None






# Decorators

def validate(**vkargs):
    """
    Validates and manipulates keyword arguments by user defined callables.
    Handles ValueError and missing arguments by raising HTTPError(403).
    """
    def decorator(func):
        def wrapper(**kargs):
            for key, value in vkargs.iteritems():
                if key not in kargs:
                    abort(403, 'Missing parameter: %s' % key)
                try:
                    kargs[key] = value(kargs[key])
                except ValueError, e:
                    abort(403, 'Wrong parameter format for: %s' % key)
            return func(**kargs)
        return wrapper
    return decorator


def route(url, **kargs):
    """
    Decorator for request handler. Same as add_route(url, handler, **kargs).
    """
    return default_app().route(url, **kargs)

def default():
    """
    Decorator for request handler. Same as set_default(handler).
    """
    return default_app().default()

def error(code=500):
    """
    Decorator for error handler. Same as set_error_handler(code, handler).
    """
    return default_app().error(code)






# Server adapter

class WSGIAdapter(object):
    def run(self, handler):
        pass

    def __repr__(self):
        return "%s()" % (self.__class__.__name__)


class CGIServer(WSGIAdapter):
    def run(self, handler):
        from wsgiref.handlers import CGIHandler
        CGIHandler().run(handler)


class ServerAdapter(WSGIAdapter):
    def __init__(self, host='127.0.0.1', port=8080, **kargs):
        WSGIAdapter.__init__(self)
        self.host = host
        self.port = int(port)
        self.options = kargs

    def __repr__(self):
        return "%s (%s:%d)" % (self.__class__.__name__, self.host, self.port)

    def run(self, handler):
        pass


class WSGIRefServer(ServerAdapter):
    def run(self, handler):
        from wsgiref.simple_server import make_server
        srv = make_server(self.host, self.port, handler)
        srv.serve_forever()


class CherryPyServer(ServerAdapter):
    def run(self, handler):
        from cherrypy import wsgiserver
        server = wsgiserver.CherryPyWSGIServer((self.host, self.port), handler)
        server.start()


class FlupServer(ServerAdapter):
    def run(self, handler):
       from flup.server.fcgi import WSGIServer
       WSGIServer(handler, bindAddress=(self.host, self.port)).run()


class PasteServer(ServerAdapter):
    def run(self, handler):
        from paste import httpserver
        from paste.translogger import TransLogger
        app = TransLogger(handler)
        httpserver.serve(app, host=self.host, port=str(self.port))


class FapwsServer(ServerAdapter):
    """
    Extremly fast webserver using libev.
    See http://william-os4y.livejournal.com/
    Experimental ...
    """
    def run(self, handler):
        import fapws._evwsgi as evwsgi
        from fapws import base
        evwsgi.start(self.host, self.port)
        evwsgi.set_base_module(base)
        def app(environ, start_response):
            environ['wsgi.multiprocess'] = False
            result = handler(environ, start_response)
            if isinstance(result, basestring):
                # fapws doesn't handle strings correctly
                return iter(result)
            else:
                return result
        evwsgi.wsgi_cb(('',app))
        evwsgi.run()


def run(app=None, server=WSGIRefServer, host='127.0.0.1', port=8080,
        interval=1, reloader=False, **kargs):
    """ Runs bottle as a web server. """
    if not app:
        app = default_app()

    quiet = bool(kargs.get('quiet', False))

    # Instantiate server, if it is a class instead of an instance
    if isinstance(server, type):
        if issubclass(server, CGIServer):
            server = server()
        elif issubclass(server, ServerAdapter):
            server = server(host=host, port=port, **kargs)

    if not isinstance(server, WSGIAdapter):
        raise RuntimeError("Server must be a subclass of WSGIAdapter")

    if not quiet and isinstance(server, ServerAdapter):
        if not reloader or os.environ.get('BOTTLE_CHILD') == 'true':
            print "Bottle server starting up (using %s)..." % repr(server)
            print "Listening on http://%s:%d/" % (server.host, server.port)
            print "Use Ctrl-C to quit."
            print
        else:
            print "Bottle auto reloader starting up..."

    try:
        if reloader and interval:
            reloader_run(server, app, interval)
        else:
            server.run(app)
    except KeyboardInterrupt:
        print "Shutting Down..."


def reloader_run(server, app, interval):
    if os.environ.get('BOTTLE_CHILD') == 'true':
        # We are a child process
        files = dict()
        for module in sys.modules.values():
            file_path = getattr(module, '__file__', None)
            if file_path and os.path.isfile(file_path):
                file_split = os.path.splitext(file_path)
                if file_split[1] in ('.py', '.pyc', '.pyo'):
                    file_path = file_split[0] + '.py'
                    files[file_path] = os.stat(file_path).st_mtime
        thread.start_new_thread(server.run, (app,))
        while True:
            time.sleep(interval)
            for file_path, file_mtime in files.iteritems():
                if not os.path.exists(file_path):
                    print "File changed: %s (deleted)" % file_path
                elif os.stat(file_path).st_mtime > file_mtime:
                    print "File changed: %s (modified)" % file_path
                else: continue
                print "Restarting..."
                app.serve = False
                time.sleep(interval) # be nice and wait for running requests
                sys.exit(3)
    while True:
        args = [sys.executable] + sys.argv
        environ = os.environ.copy()
        environ['BOTTLE_CHILD'] = 'true'
        exit_status = subprocess.call(args, env=environ)
        if exit_status != 3:
            sys.exit(exit_status)





# Templates

class TemplateError(HTTPError):
    def __init__(self, message):
        HTTPError.__init__(self, 500, message)

class BaseTemplate(object):
    def __init__(self, template='', name=None, filename=None, lookup=[]):
        """
        Create a new template.
        If a name is provided, but no filename and no template string, the
        filename is guessed using the lookup path list.
        Subclasses can assume that either self.template or self.filename is set.
        If both are present, self.template should be used.
        """
        self.name = name
        self.filename = filename
        self.template = template
        self.lookup = lookup
        if self.name and not self.filename:
            for path in self.lookup:
                fpath = os.path.join(path, self.name+'.tpl')
                if os.path.isfile(fpath):
                    self.filename = fpath
        if not self.template and not self.filename:
            raise TemplateError('Template (%s) not found.' % self.name)
        self.prepare()

    def prepare(self):
        """
        Run preparatios (parsing, caching, ...).
        It should be possible to call this multible times to refresh a template.
        """
        raise NotImplementedError

    def render(self, **args):
        """
        Render the template with the specified local variables and return an
        iterator of strings (bytes). This must be thread save!
        """
        raise NotImplementedError


class MakoTemplate(BaseTemplate):
    output_encoding=None
    input_encoding=None
    default_filters=None
    global_variables={}

    def prepare(self):
        from mako.template import Template
        from mako.lookup import TemplateLookup
        mylookup = TemplateLookup(directories=map(os.path.abspath, self.lookup)+['./'])
        if self.template:
            self.tpl = Template(self.template,
                                lookup=mylookup,
                                output_encoding=MakoTemplate.output_encoding,
                                input_encoding=MakoTemplate.input_encoding,
                                default_filters=MakoTemplate.default_filters
                                )
        else:
            self.tpl = Template(filename=self.filename,
                                lookup=mylookup,
                                output_encoding=MakoTemplate.output_encoding,
                                input_encoding=MakoTemplate.input_encoding,
                                default_filters=MakoTemplate.default_filters
                                )

    def render(self, **args):
        _defaults = MakoTemplate.global_variables.copy()
        _defaults.update(args)
        return self.tpl.render(**_defaults)


class CheetahTemplate(BaseTemplate):
    def prepare(self):
        from Cheetah.Template import Template
        self.context = threading.local()
        self.context.vars = {}
        if self.template:
            self.tpl = Template(source=self.template, searchList=[self.context.vars])
        else:
            self.tpl = Template(file=self.filename, searchList=[self.context.vars])

    def render(self, **args):
        self.context.vars.update(args)
        out = str(self.tpl)
        self.context.vars.clear()
        return [out]


class Jinja2Template(BaseTemplate):
    env = None # hopefully, a Jinja environment is actually thread-safe

    def prepare(self):
        if not self.env:
            from jinja2 import Environment, FunctionLoader
            self.env = Environment(line_statement_prefix="#", loader=FunctionLoader(self.loader))
        if self.template:
            self.tpl = self.env.from_string(self.template)
        else:
            self.tpl = self.env.get_template(self.filename)

    def render(self, **args):
        return self.tpl.render(**args).encode("utf-8")

    def loader(self, name):
        if not name.endswith(".tpl"):
            for path in self.lookup:
                fpath = os.path.join(path, name+'.tpl')
                if os.path.isfile(fpath):
                    name = fpath
                    break
        f = open(name)
        try: return f.read()
        finally: f.close()


class SimpleTemplate(BaseTemplate):
    re_python = re.compile(r'^\s*%\s*(?:(if|elif|else|try|except|finally|for|'
                            'while|with|def|class)|(include|rebase)|(end)|(.*))')
    re_inline = re.compile(r'\{\{(.*?)\}\}')
    dedent_keywords = ('elif', 'else', 'except', 'finally')

    def prepare(self):
        if self.template:
            code = self.translate(self.template)
            self.co = compile(code, '<string>', 'exec')
        else:
            code = self.translate(open(self.filename).read())
            self.co = compile(code, self.filename, 'exec')

    def translate(self, template):
        indent = 0
        strbuffer = []
        code = []
        self.includes = dict()
        class PyStmt(str):
            def __repr__(self): return 'str(' + self + ')'
        def flush(allow_nobreak=False):
            if len(strbuffer):
                if allow_nobreak and strbuffer[-1].endswith("\\\\\n"):
                    strbuffer[-1]=strbuffer[-1][:-3]
                code.append(' ' * indent + "_stdout.append(%s)" % repr(''.join(strbuffer)))
                code.append((' ' * indent + '\n') * len(strbuffer)) # to preserve line numbers
                del strbuffer[:]
        for line in template.splitlines(True):
            m = self.re_python.match(line)
            if m:
                flush(allow_nobreak=True)
                keyword, subtpl, end, statement = m.groups()
                if keyword:
                    if keyword in self.dedent_keywords:
                        indent -= 1
                    code.append(" " * indent + line[m.start(1):])
                    indent += 1
                elif subtpl:
                    tmp = line[m.end(2):].strip().split(None, 1)
                    if not tmp:
                      code.append(' ' * indent + "_stdout.extend(_base)\n")
                    else:
                      name = tmp[0]
                      args = tmp[1:] and tmp[1] or ''
                      if name not in self.includes:
                        self.includes[name] = SimpleTemplate(name=name, lookup=self.lookup)
                      if subtpl == 'include':
                        code.append(' ' * indent +
                                    "_ = _includes[%s].execute(_stdout, %s)\n"
                                    % (repr(name), args))
                      else:
                        code.append(' ' * indent +
                                    "_tpl['_rebase'] = (_includes[%s], dict(%s))\n"
                                    % (repr(name), args))
                elif end:
                    indent -= 1
                    code.append(' ' * indent + '#' + line[m.start(3):])
                elif statement:
                    code.append(' ' * indent + line[m.start(4):])
            else:
                splits = self.re_inline.split(line) # text, (expr, text)*
                if len(splits) == 1:
                    strbuffer.append(line)
                else:
                    flush()
                    for i in range(1, len(splits), 2):
                        splits[i] = PyStmt(splits[i])
                    splits = [x for x in splits if bool(x)]
                    code.append(' ' * indent + "_stdout.extend(%s)\n" % repr(splits))
        flush()
        return ''.join(code)

    def execute(self, stdout, **args):
        args['_stdout'] = stdout
        args['_includes'] = self.includes
        args['_tpl'] = args
        eval(self.co, args)
        if '_rebase' in args:
            subtpl, args = args['_rebase']
            args['_base'] = stdout[:] #copy stdout
            del stdout[:] # clear stdout
            return subtpl.execute(stdout, **args)
        return args
    def render(self, **args):
        """ Render the template using keyword arguments as local variables. """
        stdout = []
        self.execute(stdout, **args)
        return stdout



def template(tpl, template_adapter=SimpleTemplate, **args):
    '''
    Get a rendered template as a string iterator.
    You can use a name, a filename or a template string as first parameter.
    '''
    lookup = args.get('template_lookup', TEMPLATE_PATH)
    if tpl not in TEMPLATES or DEBUG:
        if "\n" in tpl or "{" in tpl or "%" in tpl or '$' in tpl:
            TEMPLATES[tpl] = template_adapter(template=tpl, lookup=lookup)
        elif '.' in tpl:
            TEMPLATES[tpl] = template_adapter(filename=tpl, lookup=lookup)
        else:
            TEMPLATES[tpl] = template_adapter(name=tpl, lookup=lookup)
    if not TEMPLATES[tpl]:
        abort(500, 'Template (%s) not found' % tpl)
    args['abort'] = abort
    args['request'] = request
    args['response'] = response
    return TEMPLATES[tpl].render(**args)


def mako_template(tpl_name, **kargs):
    kargs['template_adapter'] = MakoTemplate
    return template(tpl_name, **kargs)

def cheetah_template(tpl_name, **kargs):
    kargs['template_adapter'] = CheetahTemplate
    return template(tpl_name, **kargs)

def jinja2_template(tpl_name, **kargs):
    kargs['template_adapter'] = Jinja2Template
    return template(tpl_name, **kargs)

def view(tpl_name, **defaults):
    ''' Decorator: Rendes a template for a handler.
        Return a dict of template vars to fill out the template.
    '''
    def decorator(func):
        def wrapper(**kargs):
            out = func(**kargs)
            defaults.update(out)
            return template(tpl_name, **defaults)
        return wrapper
    return decorator

def mako_view(tpl_name, **kargs):
    kargs['template_adapter'] = MakoTemplate
    return view(tpl_name, **kargs)

def cheetah_view(tpl_name, **kargs):
    kargs['template_adapter'] = CheetahTemplate
    return view(tpl_name, **kargs)

def jinja2_view(tpl_name, **kargs):
    kargs['template_adapter'] = Jinja2Template
    return view(tpl_name, **kargs)







# Database

class BottleBucket(object):
    """ Memory-caching wrapper around anydbm """
    def __init__(self, name):
        self.__dict__['name'] = name
        self.__dict__['db'] = dbm.open(DB_PATH + '/%s.db' % name, 'c')
        self.__dict__['mmap'] = {}

    def __getitem__(self, key):
        if key not in self.mmap:
            self.mmap[key] = pickle.loads(self.db[key])
        return self.mmap[key]

    def __setitem__(self, key, value):
        if not isinstance(key, str): raise TypeError("Bottle keys must be strings")
        self.mmap[key] = value

    def __delitem__(self, key):
        if key in self.mmap:
            del self.mmap[key]
        del self.db[key]

    def __getattr__(self, key):
        try: return self[key]
        except KeyError: raise AttributeError(key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try: del self[key]
        except KeyError: raise AttributeError(key)

    def __iter__(self):
        return iter(self.ukeys())

    def __contains__(self, key):
        return key in self.ukeys()

    def __len__(self):
        return len(self.ukeys())

    def keys(self):
        return list(self.ukeys())

    def ukeys(self):
      return set(self.db.keys()) | set(self.mmap.keys())

    def save(self):
        self.close()
        self.__init__(self.name)

    def close(self):
        for key in self.mmap:
            pvalue = pickle.dumps(self.mmap[key], pickle.HIGHEST_PROTOCOL)
            if key not in self.db or pvalue != self.db[key]:
                self.db[key] = pvalue
        self.mmap.clear()
        if hasattr(self.db, 'sync'):
            self.db.sync()
        if hasattr(self.db, 'close'):
            self.db.close()

    def clear(self):
        for key in self.db:
            del self.db[key]
        self.mmap.clear()

    def update(self, other):
        self.mmap.update(other)

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            if default:
                return default
            raise


class BottleDB(threading.local):
    """ Holds multible BottleBucket instances in a thread-local way. """
    def __init__(self):
        self.__dict__['open'] = {}

    def __getitem__(self, key):
        if key not in self.open and not key.startswith('_'):
            self.open[key] = BottleBucket(key)
        return self.open[key]

    def __setitem__(self, key, value):
        if isinstance(value, BottleBucket):
            self.open[key] = value
        elif hasattr(value, 'items'):
            if key not in self.open:
                self.open[key] = BottleBucket(key)
            self.open[key].clear()
            for k, v in value.iteritems():
                self.open[key][k] = v
        else:
            raise ValueError("Only dicts and BottleBuckets are allowed.")

    def __delitem__(self, key):
        if key not in self.open:
            self.open[key].clear()
            self.open[key].save()
            del self.open[key]

    def __getattr__(self, key):
        try: return self[key]
        except KeyError: raise AttributeError(key)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try: del self[key]
        except KeyError: raise AttributeError(key)

    def save(self):
        self.close()
        self.__init__()

    def close(self):
        for db in self.open:
            self.open[db].close()
        self.open.clear()






# Modul initialization and configuration

DB_PATH = './'
TEMPLATE_PATH = ['./', './views/']
TEMPLATES = {}
DEBUG = False
HTTP_CODES = {
    100: 'CONTINUE',
    101: 'SWITCHING PROTOCOLS',
    200: 'OK',
    201: 'CREATED',
    202: 'ACCEPTED',
    203: 'NON-AUTHORITATIVE INFORMATION',
    204: 'NO CONTENT',
    205: 'RESET CONTENT',
    206: 'PARTIAL CONTENT',
    300: 'MULTIPLE CHOICES',
    301: 'MOVED PERMANENTLY',
    302: 'FOUND',
    303: 'SEE OTHER',
    304: 'NOT MODIFIED',
    305: 'USE PROXY',
    306: 'RESERVED',
    307: 'TEMPORARY REDIRECT',
    400: 'BAD REQUEST',
    401: 'UNAUTHORIZED',
    402: 'PAYMENT REQUIRED',
    403: 'FORBIDDEN',
    404: 'NOT FOUND',
    405: 'METHOD NOT ALLOWED',
    406: 'NOT ACCEPTABLE',
    407: 'PROXY AUTHENTICATION REQUIRED',
    408: 'REQUEST TIMEOUT',
    409: 'CONFLICT',
    410: 'GONE',
    411: 'LENGTH REQUIRED',
    412: 'PRECONDITION FAILED',
    413: 'REQUEST ENTITY TOO LARGE',
    414: 'REQUEST-URI TOO LONG',
    415: 'UNSUPPORTED MEDIA TYPE',
    416: 'REQUESTED RANGE NOT SATISFIABLE',
    417: 'EXPECTATION FAILED',
    500: 'INTERNAL SERVER ERROR',
    501: 'NOT IMPLEMENTED',
    502: 'BAD GATEWAY',
    503: 'SERVICE UNAVAILABLE',
    504: 'GATEWAY TIMEOUT',
    505: 'HTTP VERSION NOT SUPPORTED',
}

HTTP_ERROR_TEMPLATE = """
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html>
    <head>
        <title>Error %(status)d: %(error_name)s</title>
    </head>
    <body>
        <h1>Error %(status)d: %(error_name)s</h1>
        <p>Sorry, the requested URL <tt>%(url)s</tt> caused an error:</p>
        <pre>
            %(error_message)s
        </pre>
    </body>
</html>
"""

TRACEBACK_TEMPLATE = """
<h2>Traceback:</h2>
<pre>
%s
</pre>
"""

request = Request()
response = Response()
db = BottleDB()
local = threading.local()

#TODO: Global and app local configuration (debug, defaults, ...) is a mess

def debug(mode=True):
    global DEBUG
    DEBUG = bool(mode)

def optimize(mode=True):
    default_app().optimize = bool(mode)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2009, Rectorate of the University of Freiburg
# Copyright (c) 2009-2010, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
=============================================================================
**DataContainer** -- A Pyphant module for *self-explanatory scientific data*
=============================================================================

The *DataContainer* is Pypahnt's preferred data exchange class. It is
designed to maximize the interoperability of the various workers
provided by Pyphant.

It can be seen as an interface for exchanging data between workers and
visualizers and among workers. It reproduces the self-descriptiveness
of the *network Common Data Form* (netCDF). Once sealed it is
immutable. It can be identified by its *emd5* attribute, a unique
identifier composed of information about the origin of the container.

There are two kinds of DataContainers:

     - L{FieldContainer}
        - is designed to store *sampled scalar Fields*

     - L{SampleContainer}
        - is designed to store *tabular data*


**SampleContainer** -- A pyphant module storing tabular data
=============================================================

The *SampleContainer* combines different FieldContainers that have the
same number of sample points to a table-like representation. It stores
different observations on the same subject per row whereby each column
comprises a quantity of the same kind. Each row can be regarded as the
realization of a random variable.
"""
from __future__ import with_statement


import copy, hashlib, threading, numpy, StringIO
import urlparse
from pyphant.core.AstTransformers import (
    ReplaceName, ReplaceCompare, ReplaceOperator, UnitCalculator,
    checkDimensions)
import Helpers
import logging
_logger = logging.getLogger("pyphant")


#Default string encoding
enc = lambda s: unicode(s, "utf-8")

def parseId(id):
    u"""Returns tuple (HASH, TYPESTRING) from given .id attribute."""
    resUri = urlparse.urlsplit(id)
    return resUri[2].split('/')[-1].split('.') #(hash, uriType)


class DataContainer(object):
    u"""DataContainer \t- Base class for self-explanatory scientific data
\nDataContainer presents the following attributes:
\t  .longname \t- Notation of the data, e.g. 'electric field',
\t\t\t  which is used for the automatic annotation of charts.
\t  .shortname \t- Symbol of the physical variable in LaTeX notation,
\t\t\t  e.g. 'E_\\alpha',
\t\t\t  which is also used for the automatic annotation of charts.
\t  .id \t\t- Identifier of Enhanced MD5 (emd5) format
\t\t\t\temd5://NODE/USER/DATETIME/MD5-HASH.TYPESTRING
\t\t\t  which is set by calling method .seal() and
\t\t\t  indicates that the stored information are unchangable.
\t  .label\t- Typical axis description composed from the meta
\t\t\t  information of the DataContainer.
    """
    id = None
    hash = None
    masterLock = threading.Lock()
    def __init__(self, longname, shortname, attributes=None):
        self.longname = longname
        self.shortname = shortname
        self.machine = Helpers.getMachine()
        self.creator = Helpers.getUsername()
        if type(attributes) == type({}):
            self.attributes = attributes
        else:
            self.attributes = {}

    def appendSubscript(self, index, persistent=True):
        pos = self.shortname.find('_')
        if pos == -1:
            subscript = '_{%s}' % index
            result = self.shortname + subscript
        else:
            subscript = '_{%s,%s}' % (self.shortname[pos+1:], index)
            result = self.shortname[:pos]+subscript
        if persistent:
            self.shortname = result
        return result

    def _getLock(self):
        try:
            return self._lock
        except AttributeError, e:
            self.masterLock.acquire()
            if not hasattr(self, "_lock"):
                super(DataContainer, self).__setattr__("_lock",
                                                       threading.RLock())
            self.masterLock.release()
            return self._lock
    lock = property(_getLock)

    def __getstate__(self):
        dict = copy.copy(self.__dict__)
        del dict['_lock']
        return dict

    def __setattr__(self, attr, value):
        self.lock.acquire()
        if not self.id:
            super(DataContainer, self).__setattr__(attr, value)
        else:
            raise TypeError(
                "This DataContainer has been sealed and cannot"
                "be modified anymore.")
        self.lock.release()

    def generateHash(self, m=None):
        if m == None:
            m = hashlib.md5()
        m.update(self.longname)
        m.update(self.shortname)
        m.update(self.machine)
        m.update(self.creator)
        m.update(str(self.attributes))
        return enc(m.hexdigest())

    def seal(self, id=None):
        with self.lock:
            if self.id:
                if id and id != self.id:
                    raise ValueError('Illegal Id "%s" given. Old Id is: "%s".'
                                     % (id, self.id))
            elif id:
                self.hash, uriType = parseId(id)
                self.id = id
            else:
                self.hash = self.generateHash()
                self.timestamp = Helpers.getModuleUniqueTimestamp()
                self.id = u"emd5://%s/%s/%s/%s.%s" % (self.machine,
                                                      self.creator,
                                                      enc(self.timestamp.isoformat('_')),
                                                      self.hash,
                                                      self.typeString)


from FieldContainer import *


class SampleContainer(DataContainer):
    u"""SampleContainer(columns, longname='Realizations of random variable',
\t\t\t  shortname='X')
\t  Class of tables storing realizations of random variables as recarray
\t  colums\t: List of FieldContainer instances, each one holding a vector of all
\t\t\t  realizations of one element of the random variable.

\t  .data \t- Table of samples stored in a numpy.ndarray.
\t  .desc \t- Description numpy.dtype of the ndarray.
\t  .units \t- List of quantities objects denoting the units of
\t\t\t  the columns.
\t  .longname \t- Notation of the data, e.g. 'database query',
\t\t\t  which is used for the automatic annotation of charts.
\t  .shortname \t- Symbol of the physical variable in LaTeX notation,
\t\t\t  e.g. 'X_\\alpha',
\t\t\t  which is also used for the automatic annotation of charts.
\t  .id \t\t- Identifier of Enhanced MD5 (emd5) format
\t\t\t\temd5://NODE/USER/DATETIME/MD5-HASH.TYPESTRING
\t\t\t  which is set by calling method .seal() and
\t\t\t  indicates that the stored information are unchangable.
\t  .label\t- Typical axis description composed from the meta information
\t\t\t  of the DataContainer.
"""
    typeString = u"sample"
    def __init__(self, columns, longname='Realizations of random variable',
                 shortname='X', attributes=None):
        """columns: List of FieldContainer"""
        DataContainer.__init__(self, longname, shortname, attributes)
        self._setColumns(columns)

    def _setColumns(self, columns):
        self._columns = columns
        self.longnames = {}
        self.shortnames = {}
        for i in xrange(len(self.columns)):
            self.longnames[self.columns[i].longname] = columns[i]
            self.shortnames[self.columns[i].shortname] = columns[i]
    columns = property(lambda self:self._columns, _setColumns)

    def _getLabel(self):
        return u"%s %s" % (self.longname, self.shortname)
    label = property(_getLabel)

    def _getRawDataBytes(self):
        return [column.rawDataBytes for column in self.columns]
    rawDataBytes = property(_getRawDataBytes)

    def generateHash(self, m=None):
        if m == None:
            m = hashlib.md5()
        super(SampleContainer, self).generateHash(m)
        m.update(u''.join([c.hash for c in self.columns]))
        return enc(m.hexdigest())

    def __deepcopy__(self, memo):
        self.lock.acquire()
        res = SampleContainer.__new__(SampleContainer)
        res.columns = copy.deepcopy(self.columns, memo)
        res.longname = copy.deepcopy(self.longname, memo)
        res.shortname = copy.deepcopy(self.shortname, memo)
        res.creator = copy.deepcopy(self.creator, memo)
        res.machine = copy.deepcopy(self.machine, memo)
        res.attributes = copy.deepcopy(self.attributes, memo)
        self.lock.release()
        return res

    def seal(self, id=None):
        self.lock.acquire()
        [c.seal(c.id) for c in self.columns]
        super(SampleContainer, self).seal(id)
        self.lock.release()

    def __getitem__(self, key):
        if isinstance(key, int) or isinstance(key, slice):
            return self.columns[key]
        try:
            return self.longnames[key]
        except KeyError:
            pass
        try:
            return self.shortnames[key]
        except KeyError:
            pass
        raise KeyError(u'No column named "%s" could be found.'%key)

    def __eq__(self, other):
        try:
            if self.longname != other.longname:
                return False
            if self.shortname != other.shortname:
                return False
            if self.attributes != other.attributes:
                return False
            for selfDim, otherDim in zip(self.columns, other.columns):
                if selfDim != otherDim:
                    return False
        except:
            return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def numberOfColumns(self):
        return len(self.columns)

    def calcColumn(self, exprStr, shortname, longname):
        """
        Return an unsealed FieldContainer generated from `exprStr`.

        Parameters
        ----------
        exprStr : str
            Expression has to be a string. Specifies the mathematical
            operations which will be evaluated to generate a new FieldContainer
            containing the result. `exprStr` may contain FieldContainers,
            PhysicalQuantities, Booleans and Numbers as well as the operators
            listed below.

        shortname : str
            The `shortname` of the returned FieldContainer.

        longname : str
            The `longname`of the returned FieldContainer.

        Syntax
        ------
        FieldContainer
            FieldContainers can be adressed by their `shortname`or `longname`.
            For a FieldContainer with shortname="exampleFC" the syntax within
            `exprStr` is:
                col("exampleFC")
            where `col` stands for column and may be `Col` or `COL` as well.
            Adressing by `longname` works analogously.

        PhysicalQuantity
            Within the expression PhysicalQuantities have to be enquoted:
                "10kg", "100 m", "5 kg * m / s ** 2"
            where the whitespaces are optional.

        Booleans and Numbers
            Can just be used without quotes or braces within `exprStr`:
                True, False, 1.2, 10

        Operators
        ---------
        A list of all implemented operations that can be used within `exprStr`
        sorted by precedence from lowest precedence (least binding) to highest
        precedence (most binding):

        or
        and
        not
        Comparisons: <, <=, >, >=, <>, !=, ==
        Addition and Subtraction: +, -
        Multiplication, Division: *, /
        Positive, Negative: +x, -x

        Not implemented: ~, &, |, **, //, %, <<, >>

        Examples
        --------
        Some examples of valid expressions will be given.
        Data:
            distance = FieldContainer(scipy.array([5., 10., 1.]),
                                    Quantity('1.0 m'),
                                    longname=u"Distance",
                                    shortname=u"s")
            time = FieldContainer(scipy.array([3., 4., 5.]),
                                    Quantity('1.0 s'),
                                    longname=u"Time",
                                    shortname=u"t")
        Examplary expressions:
            exprStr = "col('Distance') / col('Time')"
            exprStr = "col('Distance') - '1 m'"
            exprStr = "col('t') >= '4 s'"
            exprStr = "col('s') > '1 m' and COL('Time') == '3s'"

        """
        exprStr = exprStr or 'True'
        import ast
        rpn = ReplaceName(self)
        expr = compile(exprStr, "<calcColumn>", 'eval', ast.PyCF_ONLY_AST)
        replacedExpr = rpn.visit(expr)
        rpc = ReplaceCompare(rpn.localDict)
        factorExpr = rpc.visit(replacedExpr)
        rpo = ReplaceOperator(rpn.localDict)
        factorExpr = rpo.visit(factorExpr)
        localDict = dict([(key, value.data) \
                          for key, value in rpn.localDict.iteritems()])
        numpyDict = {'logical_and':numpy.logical_and,
                     'logical_or':numpy.logical_or,
                     'logical_not':numpy.logical_not}
        localDict.update(numpyDict)
        data = eval(compile(factorExpr, '<calcColumn>', 'eval'), {}, localDict)
        unitcalc = UnitCalculator(rpn.localDict)
        unit, dims = unitcalc.getUnitAndDim(replacedExpr)
        if dims is None:
            assert not isinstance(data, numpy.ndarray)
            for col in self.columns:
                checkDimensions(col.dimensions[0],
                                self.columns[0].dimensions[0])
            shape = self.columns[0].dimensions[0].data.shape
            if data:
                data = numpy.ones(shape, dtype=bool)
            else:
                data = numpy.zeros(shape, dtype=bool)
            dims = [self.columns[0].dimensions[0]]
        field = FieldContainer(data, unit, dimensions=dims,
                               longname=longname, shortname=shortname)
        return field

    def filter(self, exprStr, shortname='', longname=''):
        """
        Return an unsealed SampleContainer containing only those rows where
        `exprStr` was evaluated to be True. This method replaces the old
        filter method and is mostly capable of the same operations, yet the
        syntax has changed slightly.

        Parameters
        ----------
        exprStr : str
            A string with a logical expression that has to evaluate to be
            either True or False. This can be for example an inequality or
            a comparison. For all possible operations and a description of
            the syntax as well as examples see `calcColumn`.

        shortname, longname : str, default=''
            Specify the short and long name of the resulting FC.

        """
        shortname = shortname or self.shortname
        longname = longname or self.longname
        mask = self.calcColumn(exprStr, 'm', 'mask')
        assert isinstance(mask.unit, float)
        mask = mask.data
        return self.extractRows(mask, shortname, longname)

    def extractRows(self, mask, shortname, longname):
        """
        Return an unsealed SampleContainer that contains only selected rows. The
        selection is specified via the `mask`.

        Parameters
        ----------
        mask : numpy array of Boolean values
            The length of the array has to be equal to the length of the columns
            of the SampleContainer. If the value of mask[n] is
            True, the nth row is part of the result, else it is discarded.

        shortname, longname : str
            Specify the short and long name of the resulting FC.

        """
        maskedcolumns = []
        for col in self.columns:
            try:
                maskedcol = col.getMaskedFC(mask)
            except ValueError:
                raise ValueError(
                    'Column "' + col.longname + '" has not enough rows!'
                    )
            except AttributeError:
                raise AttributeError(
                    "Masking of SampleContainers as columns is not supported."
                    )
            maskedcolumns.append(maskedcol)
        #build new SampleContainer from masked columns and return it
        result = SampleContainer(maskedcolumns,
                                 longname=longname,
                                 shortname=shortname,
                                 attributes=copy.deepcopy(self.attributes))
        return result


def assertEqual(con1, con2, rtol=1e-5, atol=1e-8):
    diagnosis = StringIO.StringIO()
    testReport = logging.StreamHandler(diagnosis)
    logger = logging.getLogger("pyphant")
    logger.addHandler(testReport)
    logger.setLevel(logging.DEBUG)
    if con1.__eq__(con2, rtol, atol):
        return True
    else:
        raise AssertionError, diagnosis.getvalue()



# -*- coding: utf-8 -*-

# Copyright (c) 2008-2009,  Rectorate of the University of Freiburg
# Copyright (c) 2009-2011, Andreas W. Liehr (liehr@users.sourceforge.net)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import unittest, numpy
from pyphant.core.DataContainer import (
    FieldContainer, SampleContainer, assertEqual
    )
from pyphant.quantities import Quantity
from pyphant.quantities.ParseQuantities import str2unit
from copy import deepcopy
from pyphant.core import LoadFMF
import pkg_resources

#Estimate floating point accuracy
ACCURACY = 1.0 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.

class FieldContainerCondenseDim(unittest.TestCase):
    def setUp(self):
        self.x = numpy.linspace(0,0.9,10)
        m = numpy.meshgrid(self.x, self.x*5)
        self.valid = numpy.tile(self.x, (10,1))
        self.invalid = [
            a.squeeze() for a in numpy.vsplit(m[0] + m[1], len(m[0]))
            ]

    def testInvalid(self):
        self.assertRaises(
            AssertionError, LoadFMF.checkAndCondense, self.invalid
            )

    def testValid(self):
        result = LoadFMF.checkAndCondense(self.valid)
        numpy.testing.assert_array_equal(self.x, result)

class TestColumn2FieldContainer(unittest.TestCase):
    def testStrings(self):
        column = ['Hello', 'World']
        result = LoadFMF.column2FieldContainer('simple string', column)
        expectedResult = FieldContainer(
            numpy.array(column), longname='simple string'
            )
        assertEqual(result, expectedResult)

    def testListofStrings(self):
        column = ['World', ['Hello', 'World'], 'World']
        result = LoadFMF.column2FieldContainer('simple string', column)
        expectedResult = FieldContainer(
            numpy.array(['World', 'Hello, World', 'World']),
            longname='simple string'
            )
        assertEqual(result, expectedResult)

    def testListofStrings2(self):
        column = [['Hello', 'World'], 'World']
        result = LoadFMF.column2FieldContainer('simple string', column)
        expectedResult = FieldContainer(
            numpy.array(['Hello, World', 'World']), longname='simple string'
            )
        assertEqual(result, expectedResult)

    def testVariable(self):
        column = [
            ('T', Quantity('22.4 degC'), Quantity('0.5 degC')),
            ('T', Quantity('11.2 degC'), Quantity('0.5 degC'))
            ]
        result = LoadFMF.column2FieldContainer('temperature', column)
        expectedResult = FieldContainer(
            numpy.array([22.4, 11.2]) ,error=numpy.array([0.5, 0.5]),
            mask=numpy.array([False, False]),
            unit='1 degC', longname='temperature', shortname='T'
            )
        assertEqual(result, expectedResult)

    def testVariableWithNaN(self):
        column = [
            ('T', Quantity('22.4 degC'), Quantity('0.5 degC')),
            ('T', Quantity('11.2 degC'), None)
            ]
        result = LoadFMF.column2FieldContainer('temperature', column)
        expectedResult = FieldContainer(
            numpy.array([22.4, 11.2]), error=numpy.array([0.5, 0.0]),
            mask = numpy.array([False, False]),
            unit='1 degC', longname='temperature', shortname='T'
            )
        assertEqual(result, expectedResult)

    def testVariableFirstNaN(self):
        column = [
            ('T', 'NaN', Quantity('0.5 degC')),
            ('T', Quantity('11.2 degC'), None)
            ]
        result = LoadFMF.column2FieldContainer('temperature', column)
        expectedResult = FieldContainer(
            numpy.array([numpy.NaN, 11.2]), error=numpy.array([0.5, 0.0]),
            mask = numpy.array([True, False]),
            unit='1 degC', longname='temperature', shortname='T')
        assertEqual(result, expectedResult)

class TestDiscriminatingJouleAndImaginary(unittest.TestCase):
    """
    In order to discriminate between an imaginary number and unit Joule,
    imaginary numbers have to be indicated only by a minor capital 'j',
    while a major capital 'J' indicates the unit Joule.
    """
    def setUp(self):
        self.inputDict = {'complexJ':'1.0j', 'Joule':'1.0J'}

    def testComplexValue(self):
        """Imaginary numbers are indicated by 'j'."""
        result = LoadFMF.item2value(self.inputDict['complexJ'])
        self.assertEqual(result, (complex(self.inputDict['complexJ']), None))

    def testJouleValue1_1(self):
        """Physical quantities with unit Joule are indicated by 'J'."""
        result = LoadFMF.item2value(self.inputDict['Joule'])
        self.assertEqual(result, (Quantity(self.inputDict['Joule']), None))

    def testJouleValue1_0(self):
        """Physical quantities with unit Joule are indicated by 'J'."""
        result = LoadFMF.item2value(
            self.inputDict['Joule'], FMFversion='1.0'
            )
        self.assertEqual(result, (Quantity(self.inputDict['Joule']), None))

class TestFMFversion1_0(unittest.TestCase):
    def almostEqual(self, a, b):
        diff = a - b
        mean = 0.5 * (a + b)
        self.assertTrue(abs(diff / mean) < ACCURACY)

    def setUp(self):
        self.FMFinput = """# -*- fmf-version: 1.0; coding: utf-8 -*-
[*reference]
title: The physical constants of the Full-Metadata Format 1.0 as documented in http://arxiv.org/abs/0904.1299
creator: Andreas W. Liehr
created: 2010-03-16
place: ICE 676, Offenburg-Karlsruhe, Germany
[Mathematical and Physical Constants]
Area of unit circle:  pi = 3.1415926535897931
Speed of light: c = 299792458 m/s
Permeability of vacuum: \mu_0 = 4.e-7 pi*N/A**2
Permittivity of vacuum: \eps_0 = 1.0 1/mu0/c**2
Gravitational constant: Grav = 6.67259e-11 m**3/kg/s**2
Planck constant: hplanck = 6.6260755e-34 J*s
Planck constant / 2pi: hbar = 0.5 hplanck/pi
Elementary charge: e = 1.60217733e-19 C
Electron mass:       m_e = 9.1093897e-31 kg
Proton mass: m_p = 1.6726231e-27 kg
Avogadro number: Nav = 6.0221367e23 1/mol
Boltzmann constant: k = 1.380658e-23 J/K
[Additional constants changed from FMF version 1.0 to 1.1]
Parsec: pc = 3.08567758128E16 m
US gallon: galUS = 4 qt
Atomic mass units: amu = 1.6605402e-27 kg
[*table definitions]
table: T
mixed: M
[*data definitions: T]
String: S
Integer: I
Float with dot: Fd
Float with exponent: Fe
Complex: C
Missing Value: V_m
Infinite Value: V_i
[*data: T]
H_2	1	1.	1e1	1+0j	nan	inf
O_2	2	.2	2E1	2+.1j	NaN	INF
O 2	2	.2	2E1	2.+2j	NAN	Inf
[*data definitions: M]
String: S
Complex: C
Float: F
[*data: M]
N_2	1	2
2	1+1j	2.
"""
    def testReadSingleFile(self):
        """
        Test the correct interpretation of physical constants
        as definied in FMF version 1.0.
        """
        consts = LoadFMF.readSingleFile(
            self.FMFinput,
            "testReadSingleFile")[0].attributes[
            'Mathematical and Physical Constants'
            ]
        self.assertEqual(
            consts[u'Speed of light'][1], str2unit("1 c", FMFversion="1.0")
            )
        self.assertEqual(consts[u'Speed of light'][1], str2unit("1 c"))
        self.assertEqual(
            consts[u'Permeability of vacuum'][1],
            str2unit("1 mu0", FMFversion="1.0")
            )
        self.assertEqual(
            consts[u'Permittivity of vacuum'][1],
            str2unit("1 eps0", FMFversion="1.0")
            )
        self.assertEqual(
            consts[u'Gravitational constant'][1],
            str2unit("1 Grav", FMFversion="1.0")
            )
        self.assertTrue(
            consts[u'Planck constant'][1],
            str2unit("1 hplanck", FMFversion="1.0")
            )
        self.almostEqual(
            consts[u'Planck constant / 2pi'][1],
            str2unit("1 hbar", FMFversion="1.0")
            )
        self.almostEqual(
            consts[u'Elementary charge'][1],
            str2unit("1 e", FMFversion="1.0")
            )
        self.assertNotEqual(
            consts[u'Elementary charge'][1], str2unit("1 e"),
            "Elementary charge has been adapted to new CODATA recommendations."
            )
        self.assertEqual(
            consts[u'Electron mass'][1], str2unit("1 me", FMFversion="1.0")
            )
        self.assertNotEqual(
            consts[u'Electron mass'][1], str2unit("1 me"),
            "Electron mass has been adapted to new CODATA recommendations."
            )
        self.assertEqual(
            consts[u'Proton mass'][1], str2unit("1 mp", FMFversion="1.0")
            )
        self.assertNotEqual(
            consts[u'Proton mass'][1], str2unit("1 mp"),
            "Proton mass has been adapted to new CODATA recommendations."
            )
        self.assertEqual(
            consts[u'Avogadro number'][1], str2unit("1 Nav", FMFversion="1.0")
            )
        self.assertEqual(
            consts[u'Boltzmann constant'][1], str2unit("1 k", FMFversion="1.0")
            )
        consts = LoadFMF.readSingleFile(
            self.FMFinput,
            "testReadSingleFile")[0].attributes[
            'Additional constants changed from FMF version 1.0 to 1.1'
            ]
        self.almostEqual(
            consts[u'Parsec'][1], str2unit("1 pc", FMFversion="1.0")
            )
        self.almostEqual(
            consts[u'US gallon'][1], str2unit("1 galUS", FMFversion="1.0")
            )
        self.assertEqual(
            consts[u'Atomic mass units'][1],
            str2unit("1 amu", FMFversion="1.0")
            )


class TestFMFversion1_1(unittest.TestCase):
    def setUp(self):
        self.FMFinput = """# -*- fmf-version: 1.1; coding: utf-8 -*-
[*reference]
title: The physical constants of the Full-Metadata Format 1.1 by Riede in doi://10.1016/j.cpc.2009.11.014
creator: Andreas W. Liehr
created: 2010-03-17
place: ICE 604, Offenburg-Karlsruhe, Germany
[Mathematical and Physical Constants]
Area of unit circle:  pi = 3.1415926535897931
Speed of light: c = 299792458 m/s
Permeability of vacuum: \mu_0 = 4.e-7 pi*N/A**2
Permittivity of vacuum: \eps_0 = 1.0 1/mu0/c**2
Faraday constant: Fa = 96485.3399 C/mol
Gravitational constant: G = 6.67428e-11 m**3/kg/s**2
Planck constant: h = 6.62606896e-34 J*s
Planck constant / 2pi: hbar = 0.5 h/pi
Elementary charge: e = 1.602176487e-19 C
Electron mass:       m_e = 9.10938215e-31 kg
Proton mass: m_p = 1.672621637e-27 kg
Avogadro number: NA = 6.02214179e23 1/mol
Boltzmann constant: k = 1.3806504e-23 J/K
Rydberg constant: Ryd = 10973731.568527 1/m
[Additional constants changed from FMF version 1.0 to 1.1]
Parsec: pc = 3.0856776E16 m
US gallon: galUS = 231 inch**3
Wave-numbers/inverse cm: invcm = 1 h*c/cm
Atomic mass units: u = 1.660538782e-27 kg
[*table definitions]
table: T
mixed: M
[*data definitions: T]
String: S
Integer: I
Float with dot: Fd
Float with exponent: Fe
Complex: C
Missing Value: V_m
Infinite Value: V_i
[*data: T]
H_2	1	1.	1e1	1+0j	nan	inf
O_2	2	.2	2E1	2+.1j	NaN	INF
O 2	2	.2	2E1	2.+2j	NAN	Inf
[*data definitions: M]
String: S
Complex: C
Float: F
[*data: M]
N_2	1	2
2	1+1j	2.
"""
    def testReadSingleFile(self):
        """
        Test the correct interpretation of physical constants
        as definied in FMF version 1.1.
        """
        consts = LoadFMF.readSingleFile(
            self.FMFinput, "testReadSingleFile")[0].attributes[
            'Mathematical and Physical Constants'
            ]
        self.assertEqual(
            consts[u'Speed of light'][1], str2unit("1 c", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Permeability of vacuum'][1],
            str2unit("1 mu0", FMFversion="1.1"),
            'The values differ by %s.' % (
                consts[u'Permeability of vacuum'][1] -\
                str2unit("1 mu0", FMFversion="1.1"),
                )
            )
        self.assertEqual(
            consts[u'Permittivity of vacuum'][1],
            str2unit("1 eps0", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Gravitational constant'][1],
            str2unit("1 G", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Planck constant'][1],
            str2unit("1 h", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Planck constant / 2pi'][1],
            str2unit("1 hbar", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Elementary charge'][1], str2unit("1 e",FMFversion="1.1"),
            'The elements %s and %s do not match.' % (
                consts[u'Elementary charge'][1],
                str2unit("1 e", FMFversion="1.1")
                )
            )
        self.assertEqual(
            consts[u'Electron mass'][1], str2unit("1 me", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Proton mass'][1], str2unit("1 mp", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Avogadro number'][1], str2unit("1 NA", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Boltzmann constant'][1], str2unit("1 k",FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Rydberg constant'][1], str2unit("1 Ryd", FMFversion="1.1")
            )
        consts = LoadFMF.readSingleFile(
            self.FMFinput, "testReadSingleFile")[0].attributes[
            'Additional constants changed from FMF version 1.0 to 1.1'
            ]
        self.assertEqual(
            consts[u'Parsec'][1], str2unit("1 pc", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'US gallon'][1], str2unit("1 galUS", FMFversion="1.1")
            )
        self.assertEqual(
            consts[u'Atomic mass units'][1], str2unit("1 u", FMFversion="1.1")
            )


class Emd5ConsistencyTestCase(unittest.TestCase):
    def setUp(self):
        self.filename = pkg_resources.resource_filename(
            'pyphant', 'tests/resources/fmf/dep.fmf'
            )

    def testImportFMF(self):
        from pyphant.core.LoadFMF import loadFMFFromFile
        table = loadFMFFromFile(self.filename)
        print "Testing imported SampleContainer for consistency..."
        for column in ['y0', 'y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8']:
            self.assertEqual(table[column].dimensions[0].id,
                             table['x'].id)

    def testRegisterFMF(self):
        from pyphant.core.KnowledgeManager import KnowledgeManager
        kmanager = KnowledgeManager.getInstance()
        table_id = kmanager.registerFMF(self.filename, temporary=True)
        table = kmanager.getDataContainer(table_id)
        print "Testing registered SampleContainer for consistency..."
        for column in ['y0', 'y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8']:
            self.assertEqual(table[column].dimensions[0].id,
                             table['x'].id)

    def testRegisterFMFSummary(self):
        from pyphant.core.KnowledgeManager import KnowledgeManager
        kmanager = KnowledgeManager.getInstance()
        table_id = kmanager.registerFMF(self.filename, temporary=True)
        table = kmanager.getDataContainer(table_id)
        print "Testing registered SampleContainer summary for consistency..."
        for column in ['y0', 'y1', 'y2', 'y3', 'y4', 'y5', 'y6', 'y7', 'y8']:
            summary = kmanager.getSummary(table[column].id)
            emd5 = summary['dimensions'][0]
            self.assertEqual(emd5, table['x'].id)


class LoaderTestCase(unittest.TestCase):
    def load(self, filename):
        res = 'tests/resources/fmf/' + filename
        data = LoadFMF.loadFMFFromFile(
            pkg_resources.resource_filename('pyphant', res)
            )
        data.seal()
        return data


class PathologicalTestCase(LoaderTestCase):
    def checkExpected(self, columns, result):
        expected = SampleContainer(columns, longname='Table', shortname='T',
                                   attributes=deepcopy(result.attributes))
        expected.seal()
        self.assertEqual(result, expected)

    def testLoadOneRowDep(self):
        result = self.load('onerow_dep.fmf')
        t = FieldContainer(numpy.array([1.0]), unit=Quantity('1 s'),
                           shortname='t', longname='time')
        s = FieldContainer(numpy.array([5.0]), unit=Quantity('1 m'),
                           shortname='s', longname='distance')
        s.dimensions[0] = deepcopy(t)
        self.checkExpected([t, s], result)

    def testLoadOneRow(self):
        result = self.load('onerow.fmf')
        t = FieldContainer(numpy.array([1.0]), unit=Quantity('1 s'),
                           shortname='t', longname='time')
        s = FieldContainer(numpy.array([2.0]), unit=Quantity('1 m'),
                           shortname='s', longname='distance')
        self.checkExpected([t, s], result)

    def testLoadOneColumn(self):
        result = self.load('onecolumn.fmf')
        t = FieldContainer(numpy.array([1, 2, 3, 4]), unit=Quantity('1 s'),
                           shortname='t', longname='time')
        self.checkExpected([t], result)

    def testLoadOneValue(self):
        result = self.load('onevalue.fmf')
        t = FieldContainer(numpy.array([1.0]), unit=Quantity('1 s'),
                           shortname='t', longname='time')
        self.checkExpected([t], result)

    def testLoadMultiTable(self):
        self.load('multitable.fmf')


class CommentCharTestCase(LoaderTestCase):
    def testHash(self):
        result = self.load('hash_test.fmf')
        title = "Testfile for # checking"
        self.assertEqual(result.attributes['*reference']['title'], title)

    def testSemi(self):
        result = self.load('semi_test.fmf')
        title = "Testfile for ; checking"
        self.assertEqual(result.attributes['*reference']['title'], title)


if __name__ == "__main__":
    import sys
    import logging
    logger = logging.getLogger('pyphant')
    hdlr = logging.StreamHandler(sys.stderr)
    formatter = logging.Formatter('[%(name)s|%(levelname)s] %(message)s')
    hdlr.setFormatter(formatter)
    logger.addHandler(hdlr)
    logger.setLevel(logging.DEBUG)
    if len(sys.argv) == 1:
        unittest.main()
    else:
        suite = unittest.TestLoader().loadTestsFromTestCase(
            eval(sys.argv[1:][0]))
        unittest.TextTestRunner().run(suite)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2007, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""
The Worker module provides the Worker base class and some support.
"""

import types, logging
from pyphant.core import Connectors, Param, WorkerRegistry
import pkg_resources


class WorkerInfo(object):
##    __slots__ = ["name", "createWorker"] #not possible due to pickling restrictions.
    def __init__(self, cls):
        self.name = cls.name
        self.toolBoxName = cls.__module__.split('.')[0]
        self.createWorker = cls

def plug(returnType):
    def setPlug(_plug):
        setattr(_plug, 'isPlug', True)
        setattr(_plug, 'returnType', returnType)
        return _plug
    return setPlug


def identifyPlugs(n,cdict):
    f=cdict[n]
    return isinstance(f, types.FunctionType)\
           and getattr(f, 'isPlug', False)

class WorkerFactory(type):
    workerRegistry = WorkerRegistry.WorkerRegistry.getInstance()
    log = logging.getLogger("pyphant")
    def __init__(cls, name, bases, cdict):
        cls._plugs=[]
        for f in filter(lambda key : identifyPlugs(key, cdict), cdict):
            cls._plugs.append((f, cdict[f]))
        super(WorkerFactory, cls).__init__(name, bases, cdict)
        if cls.__name__ != 'Worker':
            WorkerFactory.workerRegistry.registerWorker(WorkerInfo(cls))

class Worker(object):
    API = 2
    VERSION = 1
    REVISION = pkg_resources.get_distribution("pyphant").version

    __metaclass__ = WorkerFactory
    _sockets=[]
    _plugs=[]
    _params=[]
    DEFAULT_ANNOTATIONS={"pos":(0,0)}
    def __init__(self, parent=None, annotations=None):
        if annotations is None:
            annotations = {}
        self._annotations = annotations
        map(lambda k: self._annotations.setdefault(
            k, Worker.DEFAULT_ANNOTATIONS[k]),
            Worker.DEFAULT_ANNOTATIONS.keys())
        self.parent=parent
        self.initSockets(self._sockets)
        self.initPlugs(self._plugs)
        self.initParams(self._params)
        self.inithook()
        if parent:
            basename=self.getParam('name').value
            for i in xrange(10000):
                try:
                    parent.addWorker(self)
                    break
                except ValueError:
                    self.getParam('name').value = basename+'_%i'%i

    def _id(self):
        if self.parent != None:
            pre = self.parent.id
        else:
            pre = ""
        if pre != "":
            return pre+"."+self.getParam('name').value
        else:
            return self.getParam('name').value
    id = property(_id)

    def inithook(self):
        pass

    def refreshParams(self, subscriber=None):
        pass

    def initPlugs(self, plugs):
        self._plugs={}
        for (name, func) in plugs:
            p = Connectors.CalculatingPlug(getattr(self, func.func_name),
                                           name, func.returnType)
            setattr(self, 'plug'+self.upperFirstLetter(name), p)
            self._plugs[name]=p

    def initSockets(self, sockets):
        self._sockets={}
        for socket in sockets:
            if isinstance(socket, tuple):
                name=socket[0]
                type=socket[1]
            else:
                name=socket
                type=Connectors.DEFAULT_DATA_TYPE
            s=Connectors.Socket(self, name, type)
            setattr(self, 'socket'+self.upperFirstLetter(name), s)
            self._sockets[name]=s

    def upperFirstLetter(self, name):
        return name[0].upper()+name[1:]

    def initParams(self, params):
        self._params={}
        self._order=[]
        self.addParam('name',Param.createParam(self,'name','Name',
                                               self.name))
        for (name, displayName, value, subtype) in params:
            p=Param.createParam(self, name, displayName, value, subtype)
            setattr(self, 'param'+self.upperFirstLetter(name), p)
            self.addParam(name, p)

    def setAnnotation(self, key, value):
        self._annotations[key]=value

    def getAnnotation(self, key):
        try:
            return self._annotations[key]
        except KeyError:
            return None

    def getAnnotations(self):
        return self._annotations

    def addParam(self, name, param):
        if name in self._params:
            raise AttributeError, "Parameter \"" + name + "\" already exists."
        self._params[name]=param
        self._order.append(name)

    def getParam(self, name):
        return self._params[name]

    def getParamList(self):
        paramList=[]
        for name in self._order:
            paramList.append(self._params[name])
        return paramList

    def registerParamListener(self, listener, param, eventType):
        self.getParam(param).registerListener(listener, eventType)

    def unregisterParamListener(self, vetoer, param, eventType):
        self.getParam(param).unregisterListener(listener, eventType)

    def invalidate(self, event=None):
        map(lambda p: p.invalidate(), self._plugs.values())

    def getSocket(self, name):
        return self._sockets[name]

    def getSockets(self):
        return self._sockets.values()

    def getPlug(self, name):
        return self._plugs[name]

    def getPlugs(self):
        return self._plugs.values()

    def findConnectorForId(self, id):
        splittedId = id.split('.',1)
        if len(splittedId)==1:
            return getattr(self, splittedId[0])
        raise ValueError, "Illegal connector id <%s>"%id

    def connectorsExternalizationStateChanged(self, connector):
        if self.parent:
            self.parent.workersConnectorStateChanged(self,connector)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2009, Rectorate of the University of Freiburg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# * Neither the name of the Freiburg Materials Research Center,
#   University of Freiburg nor the names of its contributors may be used to
#   endorse or promote products derived from this software without specific
#   prior written permission.
#
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

u"""Provides unittest classes for KnowledgeNode.
"""


import unittest
from pyphant.core.KnowledgeNode import (KnowledgeNode, RemoteKN,
                                        get_kn_autoport)
import tempfile
import os
import socket


class KnowledgeNodeTestCase(unittest.TestCase):
    def setUp(self):
        osid, filename = tempfile.mkstemp(suffix='.sqlite3', prefix='test-')
        os.close(osid)
        self.filename = filename
        ports = [8080] + range(48621, 48771)
        self.kn = get_kn_autoport(ports, start=True, web_interface=True,
                                  dbase=filename)

    def tearDown(self):
        self.kn.stop()
        os.remove(self.filename)

    def testUUID(self):
        from urllib2 import urlopen
        stream = urlopen(self.kn.url + 'uuid/')
        uuid = stream.readline()
        stream.close()
        assert uuid == self.kn.km.uuid
        assert uuid == self.kn.uuid

    def testManageRemotes(self):
        self.kn.register_remote('127.0.0.1', -1)
        dummy = RemoteKN('127.0.0.1', -1, status=2)
        assert dummy in self.kn.remotes
        self.kn.remove_remote('127.0.0.1', -1)
        assert not dummy in self.kn.remotes

    def testGetRubbish(self):
        from pyphant.core.KnowledgeManager import DCNotFoundError
        self.kn.register_remote('127.0.0.1', 48620)
        try:
            self.kn.km.getDataContainer('emd5://rubbish.field')
        except DCNotFoundError:
            pass
        try:
            self.kn.km.getDataContainer('emd5://rubbish.sample')
        except DCNotFoundError:
            pass
        try:
            self.kn.km.getDataContainer('rubbish')
        except ValueError:
            pass
        self.kn.remove_remote('127.0.0.1', 48620)


if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1:
        unittest.main()
    else:
        suite = unittest.TestLoader().loadTestsFromTestCase(
            eval(sys.argv[1:][0]))
        unittest.TextTestRunner().run(suite)
